************************ Dreadnought Round Routines ***********************

* Notes:

********************************* Macros *********************************

macro	RESET_ROUNDS
    	if	TICKS
	clr.l	clock
	clr.l	tictoc
	endif
endm

********************************* Equates *********************************

******************************* Code Section ******************************

		text
		.68000

**************************** Each Round Routines **************************

round:		jsr	cls
		jsr	wait_vbl
		
		jsr	DEMO_START
		
		jsr	shorttunnel

		clr.w	mapper_index

		sf	disable_reset
		jsr	load_level
		move.w	#GAME_LEVEL,d0
		jsr	load_level_graphics
		
		st	IN_GAME
		cmp.w	#20,level
		seq.b	endboss

		jsr	clearworld		;clear the old scene
		jsr	makeworld		;create a new scene
		jsr	shadeshapes
		jsr	load_level_music
		
		move.b	#NOMESSAGE,HUDMESSAGE
		sf	HUDFLASH
		sf	level_complete
		clr.w	level_timer
		clr.w	target_period
		clr.w	target_frame
		clr.w	target_frame
		jsr	initshapes
	  	RESET_ROUNDS

		st.b	texturesoff
		st.b	debugoff

		sf.b	WARPED

		move.w	#-1,gameoverscale

		jsr	cls
		jsr	wait_vbl
		lea	CLUT,a0
		lea	options_palette+2,a1
		moveq	#128-1,d2
.loop1:	move.l	(a1)+,(a0)+
		dbf	d2,.loop1
	
		sf	clock_on
		move.b	#9,minutes
		move.b	#0,seconds
		st	clock_on
		jsr	initalise_mapper
		st	mapper_on

	

		moveq.l	#SAY_ENTER_MATRIX,d0
		jsr	sfxplay

		clr.w	counter



		clr.l	frames
		clr.l	lastframes
	
.doround:	
		addq.w	#1,counter
	
		jsr	read_joypad

		move.l	joyedge1,d0
		or.l	joyedge2,d0

		btst	#KEY_8,d0		;toggles the textures on and off
		beq.s	.notextures
		not.b	texturesoff
.notextures:
		btst	#KEY_9,d0		;toggles the debug info on and off
		beq.s	.nodebug
		not.b	debugoff
.nodebug:
		cmp.b	#DEMO_OFF,demo_mode
		bne	.in_demo_mode

		btst	#PAUSE,d0		;check for pause
		beq	.nopause

		st	paused
		sf	clock_on
		clr.l	joyedge1
		clr.l	joyedge2
		moveq	#$3f,d0
		jsr	MUS_HALT
		jsr	MUS_DISABLE_FX
		jsr	MUS_ENABLE_FX
		jsr	create_paused_sprites
		jsr	NEW_OL

.paused:	jsr	wait_vbl
		jsr	read_joypad
		jsr	handle_paused_sprites
		move.l	joyedge1,d0
		or.l	joyedge2,d0
		btst	#PAUSE,d0
		beq.s	.paused
		clr.l	joyedge1
		clr.l	joyedge2
		st	clock_on
		jsr	MUS_RESUME
		jsr	MUS_DISABLE_FX
		jsr	MUS_ENABLE_FX
		jsr	create_eeprom_buffer_pause
		sf	paused
		bra.s	.finished_paused
.nopause:
.in_demo_mode:		
		jsr	NEW_OL			;CLS playfield/screen
.finished_paused:
		jsr	CLEAR_SFX_LIST		;ensure no sfx present
		tst.b	WARPED
		beq	.no_warped
		sf	WARPED
			
		moveq	#$30,d0
		jsr	MUS_HALT
		jsr	MUS_RESUME
		moveq	#SFX_WARP,d0
		jsr	sfxplay
.no_warped:
		move.l	clock,tictoc		
		sf	paused
		
		lea	colouranimtab,a0	;do the animated colours
		jsr	colouranim

		move.l	rasterb,rasterd		;set draw raster
		jsr	ADD_VIEWSCREEN		;add polygon playfield

		lea	world,a0		;update the world
		jsr	doitem
		jsr	waitgpu			;wait for cameras
		jsr	waitblitter

		move.l	camera_addr,a0		;update the cameras
		jsr	doitem
		jsr	waitgpu			;wait for cameras
		jsr	waitblitter


		move.l	clock,d0
		sub.l	d0,tictoc

		bsr	showdebug
		jsr	PROCESS_SFX		;sort sfx list by distance
		
		if	THREE_LISTS
		jsr	switchabc		;switch screens
		else
		jsr	switchab
		endif

		jsr	swap_hud_pointers
		jsr	game_over_show
		jsr	replay_show
		jsr	done_level

		
		tst.b	level_complete
		bne.s	.doneround

		tst.b	exit_title
		bne.s	.doneround

		;tst.w	minutes
		;beq.s	.doneround


		move.l	frames,lastframes
		clr.l	frames

		cmp.w	#48,gameoverscale
		blt	.doround

.doneround:	sf	mapper_on
		jsr	DEMO_END
		rts

showdebug:	tst.b	debugoff
		bne.s	.nodebug
		bsr	showframerate
		bsr	showobjects
		bsr	showpolygons
.nodebug:
		rts

showframerate:	move.l	clock,d1
		sub.l	oldclock,d1
		addq	#1,d1
		move.l	clock,oldclock
		move.l	#60,d0
		divu	d1,d0
		ext.l	d0
		lea	framedigits,a5
		lea	framenumber,a6
		bsr	shownumber
		rts

showobjects:	moveq.l	#0,d0
		lea	world,a6
		move.l	ifirst(a6),d7
		beq.s	.none
.count:
		addq.l	#1,d0
		move.l	d7,a6
		move.l	inext(a6),d7
		bne.s	.count
.none:
		lea	objectdigits,a5
		lea	objectnumber,a6
		bsr	shownumber
		rts

showpolygons:	move.l	numshaded,d0
		lea	shadeddigits,a5
		lea	shadednumber,a6
		bsr	shownumber

		move.l	numtextured,d0
		add.l	d0,d0
		add.l	d0,d0
		lea	textureddigits,a5
		lea	texturednumber,a6
		bsr	shownumber
		rts

shownumber:	move.l	#100,d1			;converts a number into
		moveq.l	#2,d7			;digits in the number string
.calcdigits:
		divu	d1,d0
		add.b	#"0",d0
		move.b	d0,(a5)+
		swap	d0
		ext.l	d0
		divu	#10,d1
		dbf	d7,.calcdigits

		move.l	rasterd,d0
		move.l	#PITCH4|PIXEL16|WID320|XADDPIX,d1
		jsr	print_text_routine

		rts
showsignednumber:
		move.b	#"+",d1			;show sign
		tst.l	d0
		bpl.s	.pos
		neg.l	d0
		move.b	#"-",d1
.pos:
		move.b	d1,(a5)+

		move.l	#10000,d1		;show digits
		moveq.l	#4,d7
.calcdigits:
		divu	d1,d0
		add.b	#"0",d0
		move.b	d0,(a5)+
		swap	d0
		ext.l	d0
		divu	#10,d1
		dbf	d7,.calcdigits

		move.l	rasterd,d0
		move.l	#PITCH4|PIXEL16|WID320|XADDPIX,d1
		jsr	print_text_routine
		rts



******************************* Data Section ******************************

		.data
		
********************************* Variables *******************************

		if	TICKS
tictoc:		dc.l	0
		endif

		if	START_TWO=0

one_view:	dc.w	0
camera_addr:	dc.l	cameras1,cameras2

		else

one_view:	dc.w	-1
camera_addr:	dc.l	cameras2,cameras1

		endif

oldclock:	dc.l	0

framenumber:	dc.b	PAT,0,10,0,30
framedigits:	dc.b	"000"
		dc.b	" Frames Per Second",PEND
objectnumber:	dc.b	PAT,0,10,0,38
objectdigits:	dc.b	"000"
		dc.b	" Objects In System"
		dc.b	PEND

shadednumber:	dc.b	PAT,0,10,0,46
shadeddigits:	dc.b	"000"
		dc.b	" Gouraud Shaded Polygons"
		dc.b	PEND

texturednumber:	dc.b	PAT,0,10,0,54
textureddigits:	dc.b	"000"
		dc.b	" Texture Mapped Polygons"
		dc.b	PEND



debugoff:	dc.b	0
WARPED:		dc.b	0

******************************* Colour Animations ****************************

		even

colouranimtab:	dc.l	redpulse
		dc.l	fireflash1
		dc.l	fireflash2
		dc.l	fireflash3
		dc.l	electricflash
		dc.l	0

redpulse:	dc.l	redpulse+8,animcols+0
		dc.w	$f0,$f1
		dc.w	$f2,$f3
		dc.w	$f4,$f5
		dc.w	$f6,$f7
		dc.w	$f8,$f9
		dc.w	$fa,$fb
		dc.w	$fc,$fd
		dc.w	$fe,$ff
		dc.w	$ff,$fe
		dc.w	$fd,$fc
		dc.w	$fb,$fa
		dc.w	$f9,$f8
		dc.w	$f7,$f6
		dc.w	$f5,$f4
		dc.w	$f3,$f2
		dc.w	$f1,$f0
		dc.w	-1

fireflash1:	dc.l	fireflash1+8,animcols+1
		dc.w	$f0,$f0
		dc.w	$fe,$fe
		dc.w	$f2,$f2
		dc.w	$f3,$f3
		dc.w	$fb,$fb
		dc.w	$f5,$f5
		dc.w	$fa,$fa
		dc.w	$f7,$f7
		dc.w	$fd,$fd
		dc.w	$f9,$f9
		dc.w	$f6,$f6
		dc.w	$f8,$f8
		dc.w	$fc,$fc
		dc.w	$f1,$f1
		dc.w	$f4,$f4
		dc.w	$ff,$ff
		dc.w	-1

fireflash2:	dc.l	fireflash2+8,animcols+2
		dc.w	$f3,$f3
		dc.w	$f4,$f4
		dc.w	$fb,$fb
		dc.w	$f5,$f5
		dc.w	$fa,$fa
		dc.w	$f6,$f6
		dc.w	$f7,$f7
		dc.w	$fe,$fe
		dc.w	$f2,$f2
		dc.w	$fd,$fd
		dc.w	$fc,$fc
		dc.w	$f9,$f9
		dc.w	$f8,$f8
		dc.w	$f1,$f1
		dc.w	$ff,$ff
		dc.w	$f0,$f0
		dc.w	-1

fireflash3:	dc.l	fireflash3+8,animcols+3
		dc.w	$e3,$e3
		dc.w	$e4,$e4
		dc.w	$eb,$eb
		dc.w	$e5,$e5
		dc.w	$ea,$ea
		dc.w	$e6,$e6
		dc.w	$e7,$e7
		dc.w	$ee,$ee
		dc.w	$e2,$e2
		dc.w	$ed,$ed
		dc.w	$ec,$ec
		dc.w	$e9,$e9
		dc.w	$e8,$e8
		dc.w	$e1,$e1
		dc.w	$ef,$ef
		dc.w	$e0,$e0
		dc.w	-1

electricflash:	dc.l	electricflash+8,animcols+4
		dc.w	$26
		dc.w	$37
		dc.w	$48
		dc.w	$36
		dc.w	$27
		dc.w	$46
		dc.w	$28
		dc.w	$47
		dc.w	$38
		dc.w	$48
		dc.w	$38
		dc.w	$28
		dc.w	$27
		dc.w	$46
		dc.w	$26
		dc.w	$37
		dc.w	$47
		dc.w	$36
		dc.w	-1


;slowmode:	dc.w	0
counter:	dc.w	0

   	.phrase


	.bss
level_complete:	ds.b	1
	even

