************************** Camera Control Routines ***************************

* Notes:

* The cameras for the player's death sequence
* are controlled by the player's shield value
* <0 means the player is exploding
* 0 means the player is regenerating
* >0 means the player is still safe

* So as long as these values are set correctly
* by the player's explosion routines
* the cameras should use the correct routines

********************************* Code Section *******************************

		text
		.68000

******************************** Camera Routines *****************************

******************************* Cockpit Camera *******************************

docockpitcam:	move.l	ictarget(a0),a1

		movem.l	isxcoord(a1),d0-d2
		add.l	#camheight,d1
		movem.l	d0-d2,icxcoord(a0)

		movem.w	isxx(a1),d0-d7/a2
		movem.w	d0-d7/a2,icxx(a0)

		move.l	isanimbase(a1),-(sp)	;have to clear 'isanimbase'
		move.l	a1,-(sp)		;to prevent the target
		clr.l	isanimbase(a1)		;being drawn
		bsr	showview
		move.l	(sp)+,a1
		move.l	(sp)+,isanimbase(a1)

		sf	cockpit_visible
		bsr	externalhud
		bsr	display_cross_hair

		bsr	changecam
		bsr	checkfordead
		rts

******************************* Floating Camera ******************************

dofloatingcam:	lea	icxx(a0),a6		;calculate the camera's
		movem.w	icxangle(a0),d0-d2	;orientation matrix
		bsr	anglematrix
		move.l	a6,a4			;now make that matrix
		move.l	ictarget(a0),a5		;relative to the target's
		lea	isxx(a5),a5		;matrix
		lea	-18(sp),sp
		move.l	sp,a6
		bsr	matmult
		movem.w	(sp)+,d0-d7/a6
		movem.w	d0-d7/a6,icxx(a0)

		movem.l	icxcoord(a0),d0-d2	;get old position

		movem.w	iczx(a0),d3-d5		;get desired position
		move.w	icdist(a0),d6
		neg.w	d6
		muls	d6,d3
		muls	d6,d4
		muls	d6,d5
		add.l	d3,d3
		add.l	d3,d3
		add.l	d4,d4
		add.l	d4,d4
		add.l	d5,d5
		add.l	d5,d5
		swap	d3
		swap	d4
		swap	d5
		ext.l	d3
		ext.l	d4
		ext.l	d5
		move.l	ictarget(a0),a1
		add.l	isxcoord(a1),d3
		add.l	isycoord(a1),d4
		add.l	iszcoord(a1),d5
		add.l	#camheight,d4

		sub.l	d0,d3			;move towards desired position
		sub.l	d1,d4
		sub.l	d2,d5
		asr.l	#3,d3
		asr.l	#3,d4
		asr.l	#3,d5
		add.l	d0,d3
		add.l	d1,d4
		add.l	d2,d5

		bsr	cliptovolumes		;make sure new position is valid

		movem.l	d0-d2,icxcoord(a0)	;save new position

		move.l	ictarget(a0),a1		;point the camera
		movem.l	isxcoord(a1),d0-d2	;at it's target
		movem.l	icxcoord(a0),d3-d5
		add.l	#camheight,d1
		lea	icxx(a0),a6
		bsr	pointmatrix

		bsr	showview		;display camera
		bsr	externalhud		;display hud

		bsr	rotateandzoom		;rotate and zoom camera
		bsr	changecam		;change camera mode
		bsr	checkfordead		;use dead cam if required
		rts

******************************** Fixed Camera ********************************

dofixedcam:	nop				;makes 'fixedcam' and 'closefixedcam' have different addresses for when changing cams
doclosefixedcam:
		lea	icxx(a0),a6		;calculate the camera's
		movem.w	icxangle(a0),d0-d2	;orientation matrix
		bsr	anglematrix
		move.l	a6,a4			;now make that matrix
		move.l	ictarget(a0),a5		;relative to the target's
		lea	isxx(a5),a5		;matrix
		lea	-18(sp),sp
		move.l	sp,a6
		bsr	matmult
		movem.w	(sp)+,d0-d7/a6
		movem.w	d0-d7/a6,icxx(a0)

		movem.l	icxcoord(a0),d0-d2
		movem.w	iczx(a0),d3-d5		;calculate the camera's
		move.w	icdist(a0),d6		;desired position
		neg.w	d6
		muls	d6,d3
		muls	d6,d4
		muls	d6,d5
		add.l	d3,d3
		add.l	d3,d3
		add.l	d4,d4
		add.l	d4,d4
		add.l	d5,d5
		add.l	d5,d5
		swap	d3
		swap	d4
		swap	d5
		ext.l	d3
		ext.l	d4
		ext.l	d5
		move.l	ictarget(a0),a1
		add.l	isxcoord(a1),d3
		add.l	isycoord(a1),d4
		add.l	iszcoord(a1),d5
		add.l	#camheight,d4

		bsr	cliptovolumes

		movem.l	d0-d2,icxcoord(a0)	;save new position

		move.l	ictarget(a0),a1		;point the camera
		movem.l	isxcoord(a1),d0-d2	;at it's target
		movem.l	icxcoord(a0),d3-d5
		add.l	#camheight,d1
		lea	icxx(a0),a6
		bsr	pointmatrix

		bsr	showview		;display camera
		bsr	externalhud		;display hud

		bsr	changecam
		bsr	checkfordead
		rts

******************************** Panning Camera ******************************

dopanningcam:	bsr	findcamera		;use the closest camera
		movem.l	(a2)+,d0-d2
		movem.l	d0-d2,icxcoord(a0)

		move.l	ictarget(a0),a1		;point at the target
		movem.l	isxcoord(a1),d0-d2
		movem.l	icxcoord(a0),d3-d5
		add.l	#camheight,d1
		lea	icxx(a0),a6
		bsr	pointmatrix

		bsr	showview
		bsr	externalhud

		bsr	changecam
		bsr	checkfordead
		rts

***************************** Death Sequence Camera **************************

dodeadcam:	move.w	iczoom(a0),d0	 	;zoom the camera out
		add.w	#63,d0
		cmp.w	#2048-512,d0
		ble.s	.zoomout
		move.w	#2048-512,d0
		move.l	ictarget(a0),a1
		tst.w	ioenergy(a1)
		bmi.s	.zoomout
		move.l	#doregencam,ido(a0)
.zoomout:
		move.w	d0,iczoom(a0)

		bsr	zoomedcamera		;do a zoomed camera
		rts

*************************** Regenerate Sequence Camera ***********************

doregencam:	move.l	ictarget(a0),a1		;check if the player is regenerated
		tst.w	ioenergy(a1)
		beq.s	.notyet
		move.l	idraw(a0),ido(a0)	;restore previous camera type
.notyet:
		move.w	iczoom(a0),d0	 	;zoom the camera in
		sub.w	#12,d0
		cmp.w	#512,d0
		bge.s	.zoomin
		move.w	#512,d0
.zoomin:
		move.w	d0,iczoom(a0)

		bsr	zoomedcamera		;do a zoomed camera
		rts

***************************** Camera Subroutines *****************************

zoomedcamera:	bsr	findcamera		;use the closest camera

		move.w	iczoom(a0),d0		;zoom the camera towards the target
		bsr	sincos
		move.w	d0,d6
		asr.w	d6
		add.w	#$2000,d6

		muls	#15000,d6		;so we don't go inside
		add.l	d6,d6			;the target
		add.l	d6,d6
		swap	d6

		move.l	ictarget(a0),a1
		movem.l	isxcoord(a1),d0-d2
		add.l	#camheight,d1
		movem.l	(a2),d3-d5
		sub.l	d3,d0
		sub.l	d4,d1
		sub.l	d5,d2

		muls	d6,d0
		muls	d6,d1
		muls	d6,d2
		add.l	d0,d0
		add.l	d0,d0
		add.l	d1,d1
		add.l	d1,d1
		add.l	d2,d2
		add.l	d2,d2
		swap	d0
		swap	d1
		swap	d2
		ext.l	d0
		ext.l	d1
		ext.l	d2

		add.l	d3,d0
		add.l	d4,d1
		add.l	d5,d2

		movem.l	d0-d2,icxcoord(a0)	;that's our zoomed camera

		move.l	ictarget(a0),a1		;point the camera
		movem.l	isxcoord(a1),d0-d2	;at it's target
		movem.l	icxcoord(a0),d3-d5
		lea	icxx(a0),a6
		bsr	pointmatrix

		bsr	showview
		bsr	externalhud
		rts

showview:	move.l	ictarget(a0),a1			;get target's rezz
		move.w	isrezz(a1),d0
		move.w	d0,-(sp)			;save target's rezz
		move.l	rezz,-(sp)			;save scene rezz
		ext.l	d0
		move.w	#16384,isrezz(a1)		;set object's rezz
		move.l	d0,rezz				;set scene rezz

		jsr	waitgpu
		bsr	initview

		clr.l	numshaded
		clr.l	numtextured
		move.l	#shadedbuffer,nextshaded	;calculate the view
		move.l	#texturedbuffer,nexttextured
		lea	gpushapes,a6
		jsr	copygpu
		jsr	startgpu
		jsr	waitgpu

		move.l	nextshaded,a5
		move.l	nexttextured,a6
		move.l	#0,(a5)
		move.l	#0,(a6)

		tst.b	IN_GAME
		beq.s	.not_playing
		movem.l	d0-d7/a0-a6,-(sp)		;should just clear the
		move.l	rasterd,a0			;window really

		move.l	#$00000000,d7
		jsr	clearraster

		move.l	rasterz,a0
		move.l	#$7fff7fff,d7
		jsr	clearraster
		movem.l	(sp)+,d0-d7/a0-a6
.not_playing:
		move.l	#shadedbuffer,nextshaded	;draw the shaded polygons
		lea	gpushaded,a6
		jsr	copygpu
		jsr	startgpu
		jsr	waitgpu

		move.l	rasterc,pixelmap0

		move.l	#texturedbuffer,nexttextured	;draw the textured polygons
		lea	gputextured,a6
		jsr	copygpu
		jsr	startgpu

		move.l	ictarget(a0),a1
		move.l	(sp)+,rezz			;restore scene rezz
		move.w	(sp)+,isrezz(a1)		;restore target's rezz
		rts

initview:	movem.w	icxx(a0),d0-d7/a6		;set the view orientation
		movem.l	d0-d7/a6,viewxx

		movem.l	icxcoord(a0),d0-d2		;set the view position
		movem.l	d0-d2,viewxcoord

		move.l	#30000,d3			;set the 3d clipping window
		sub.l	d3,d0
		sub.l	d3,d1
		sub.l	d3,d2
		movem.l	d0-d2,viewlx
		add.l	d3,d3
		add.l	d3,d0
		add.l	d3,d1
		add.l	d3,d2
		movem.l	d0-d2,viewhx

		movem.w	icclx(a0),d0-d5			;set the 2d clipping window
		movem.l	d0-d5,clx

		move.l	#world,shapelist		;set the shape list
		rts

findcamera:	move.l	ictarget(a0),a1			;returns the camera closest
		movem.l	isxcoord(a1),d0-d2		;to the target in a2

		lea	panningcams,a1
		move.l	#$7fffffff,d6
		move.w	numpanningcams,d7
		subq.w	#1,d7
.findclosest:
		movem.l	(a1),d3-d5
		sub.l	d0,d3
		bpl.s	.xpos
		neg.l	d3
.xpos:
		sub.l	d1,d4
		bpl.s	.ypos
		neg.l	d4
.ypos:
		sub.l	d2,d5
		bpl.s	.zpos
		neg.l	d5
.zpos:
		add.l	d4,d3
		add.l	d5,d3
		cmp.l	d3,d6
		blt.s	.nextcam
		move.l	d3,d6
		move.l	a1,a2
.nextcam:
		lea	12(a1),a1
		dbf	d7,.findclosest
.noneleft:
		rts

cliptovolumes:	move.l	ictarget(a0),a1		;if the player has warped
		tst.w	iowarped(a1)		;put the old camera position
		beq.s	.nowarp			;at the player's position
		movem.l	isxcoord(a1),d0-d2
.nowarp:
		bsr	findvolume		;get the old camera position
		move.l	a1,d7			;if it was valid
		bne.s	.wasok

		bsr	findclosestvolume	;if we were never in a valid position
		move.l	a1,a2			;find the closest volume
		bra.s	.clip			;and clip to that
.wasok:
		move.l	a1,a2

		exg.l	d0,d3			;check if the new camera
		exg.l	d1,d4			;position is valid
		exg.l	d2,d5
		bsr	findvolume
		move.l	a1,d7
		bne.s	.znothigh
.clip:
		cmp.l	(a2)+,d0		;since the camera has
		bge.s	.xnotlow		;moved outside all volumes
		move.l	-4(a2),d0		;clip to previous volume
.xnotlow:
		cmp.l	(a2)+,d1
		bge.s	.ynotlow
		move.l	-4(a2),d1
.ynotlow:
		cmp.l	(a2)+,d2
		bge.s	.znotlow
		move.l	-4(a2),d2
.znotlow:
		cmp.l	(a2)+,d0
		ble.s	.xnothigh
		move.l	-4(a2),d0
.xnothigh:
		cmp.l	(a2)+,d1
		ble.s	.ynothigh
		move.l	-4(a2),d1
.ynothigh:
		cmp.l	(a2)+,d2
		ble.s	.znothigh
		move.l	-4(a2),d2
.znothigh:
		rts

findvolume:	sub.l	a1,a1			;find which volume
		lea	volumes,a6		;d0,d1,d2 is inside
		move.w	numvolumes,d6
		subq.w	#1,d6
.find:
		cmp.l	(a6)+,d0
		blt.s	.outside1
		cmp.l	(a6)+,d1
		blt.s	.outside2
		cmp.l	(a6)+,d2
		blt.s	.outside3
		cmp.l	(a6)+,d0
		bgt.s	.outside4
		cmp.l	(a6)+,d1
		bgt.s	.outside5
		cmp.l	(a6)+,d2
		bgt.s	.outside6
		lea	-24(a6),a1
		bra.s	.noneleft
.outside1:
		lea	20(a6),a6
		dbf	d6,.find
		bra.s	.noneleft
.outside2:
		lea	16(a6),a6
		dbf	d6,.find
		bra.s	.noneleft
.outside3:
		lea	12(a6),a6
		dbf	d6,.find
		bra.s	.noneleft
.outside4:
		addq.l	#8,a6
		dbf	d6,.find
		bra.s	.noneleft
.outside5:
		addq.l	#4,a6
		dbf	d6,.find
		bra.s	.noneleft
.outside6:
		dbf	d6,.find
.noneleft:
		rts

findclosestvolume:
		lea	volumes,a6		;find which volume is
		move.l	#$7fffffff,d6		;closest to d0,d1,d2
		move.w	numvolumes,d7
		subq.w	#1,d7
.find:
		move.l	a6,a5
		movem.l	(a5)+,d3-d5
		add.l	(a5)+,d3
		add.l	(a5)+,d4
		add.l	(a5)+,d5
		asr.l	d3
		asr.l	d4
		asr.l	d5
		sub.l	d0,d3
		bpl.s	.xpos
		neg.l	d3
.xpos:
		sub.l	d1,d4
		bpl.s	.ypos
		neg.l	d4
.ypos:
		sub.l	d2,d5
		bpl.s	.zpos
		neg.l	d5
.zpos:
		add.l	d4,d3
		add.l	d5,d3
		cmp.l	d3,d6
		ble.s	.nope
		move.l	d3,d6
		move.l	a6,a1
.nope:
		lea	24(a6),a6
		dbf	d7,.find
		rts

rotateandzoom:	move.l	iccontroller(a0),a1	;rotate and zoom the
		move.l	(a1),d0			;camera with the joypad

		movem.w	icxrot(a0),d1-d2
		move.w	iczoom(a0),d3

		btst	#KEY_1,d0
		beq.s	.noup
		subq.w	#4,d1
		cmp.w	#-40,d1
		bge.s	.nodown
		moveq.l	#-40,d1
		bra.s	.nodown
.noup:
		btst	#KEY_4,d0
		beq.s	.nodown
		addq.w	#4,d1
		cmp.w	#40,d1
		ble.s	.nodown
		moveq.l	#40,d1
.nodown:
		btst	#KEY_2,d0
		beq.s	.noleft
		subq.w	#4,d2
		cmp.w	#-40,d2
		bge.s	.noright
		moveq.l	#-40,d2
		bra.s	.noright
.noleft:
		btst	#KEY_5,d0
		beq.s	.noright
		addq.w	#4,d2
		cmp.w	#40,d2
		ble.s	.noright
		moveq.l	#40,d2
.noright:
		btst	#KEY_3,d0
		beq.s	.nozin
		subq.w	#8,d3
		cmp.w	#-60,d3
		bge.s	.nozout
		moveq.l	#-60,d3
		bra.s	.nozout
.nozin:
		btst	#KEY_6,d0
		beq.s	.nozout
		addq.w	#8,d3
		cmp.w	#60,d3
		ble.s	.nozout
		moveq.l	#60,d3
.nozout:

		tst.w	d1			;slow down the rotate
		beq.s	.xrotok			;and zoom until it stops
		bmi.s	.xrotn
.xrotp:
		subq.w	#2,d1
		bpl.s	.xrotok
		moveq.l	#0,d1
		bra.s	.xrotok
.xrotn:
		addq.w	#2,d1
		ble.s	.xrotok
		moveq.l	#0,d1
.xrotok:
		tst.w	d2
		beq.s	.yrotok
		bmi.s	.yrotn
.yrotp:
		subq.w	#2,d2
		bpl.s	.yrotok
		moveq.l	#0,d2
		bra.s	.yrotok
.yrotn:
		addq.w	#2,d2
		ble.s	.yrotok
		moveq.l	#0,d2
.yrotok:
		tst.w	d3
		beq.s	.zok
		bmi.s	.zn
.zp:
		subq.w	#2,d3
		bpl.s	.zok
		moveq.l	#0,d3
		bra.s	.zok
.zn:
		addq.w	#2,d3
		ble.s	.zok
		moveq.l	#0,d3
.zok:
		movem.w	d1-d2,icxrot(a0)		;update the camera
		add.w	icxangle(a0),d1			;angles
		add.w	icyangle(a0),d2
		move.w	#2047,d5
		and.w	d5,d1
		and.w	d5,d2
		movem.w	d1-d2,icxangle(a0)

		move.w	d3,iczoom(a0)			;update the camera
		add.w	icdist(a0),d3			;distance
		cmp.w	#mincamdist,d3
		blt.s	.dlow
		cmp.w	#maxcamdist,d3
		ble.s	.dok
.dhigh:
		move.w	#maxcamdist,d3
		bra.s	.dok
.dlow:
		move.w	#mincamdist,d3
.dok:
		move.w	d3,icdist(a0)
		rts

changecam:	tst.b	usecam			;check if we want to force
		beq.s	.noset			;a particular camera

		cmp.b	#cockpitcam,usecam
		beq.s	.cam1
		cmp.b	#floatingcam,usecam
		beq.s	.cam2
		cmp.b	#fixedcam,usecam
		beq.s	.cam3
		cmp.b	#closefixedcam,usecam
		beq	.cam4
		bra	.cam5
.noset:
		tst.b	nochangecam
		bne	.nochange

		move.l	iccontroller(a0),a1	;check for changing the camera
		move.l	4(a1),d0

		move.l	ido(a0),d7
		btst	#KEY_1,d0		;switch to the cockpit camera
		beq.s	.nocam1
.cam1:
		move.l	#docockpitcam,d7
		moveq.l	#VIEWMODE1,d1
		bra	.nocam5
.nocam1:
		btst	#KEY_2,d0		;switch to the floating camera
		beq.s	.nocam2
.cam2:
		move.l	#dofloatingcam,d7
		moveq.l	#VIEWMODE2,d1
		move.w	#camxangle,icxangle(a0)
		move.w	#camyangle,icyangle(a0)
		move.w	#camzangle,icyangle(a0)
		move.w	#maxcamdist,icdist(a0)
		bra.s	.nocam5
.nocam2:
		btst	#KEY_3,d0		;switch to the fixed camera
		beq.s	.nocam3
.cam3:
		move.l	#dofixedcam,d7
		moveq.l	#VIEWMODE3,d1
		move.w	#camxangle,icxangle(a0)
		move.w	#camyangle,icyangle(a0)
		move.w	#camzangle,icyangle(a0)
		move.w	#midcamdist,icdist(a0)
		bra.s	.nocam5
.nocam3:
		btst	#KEY_4,d0		;switch to the close fixed camera
		beq.s	.nocam4
.cam4:
		move.l	#doclosefixedcam,d7
		moveq.l	#VIEWMODE4,d1
		move.w	#camxangle,icxangle(a0)
		move.w	#camyangle,icyangle(a0)
		move.w	#camzangle,icyangle(a0)
		move.w	#mincamdist,icdist(a0)
		bra.s	.nocam5
.nocam4:
		btst	#KEY_5,d0		;switch to the panning camera
		beq.s	.nochange
.cam5:
		move.l	#dopanningcam,d7
		moveq.l	#VIEWMODE5,d1
.nocam5:
		move.l	d7,ido(a0)		;set the new camera
		move.b	d1,HUDMESSAGE
		move.w	#cameramessagetime,HUDTIMER
		sf.b	HUDFLASH
		sf.b	usecam
.nochange:
		rts

checkfordead:	move.l	ictarget(a0),a1		;if the player is dead
		tst.w	ioenergy(a1)   		;set up for dead camera
		bgt.s	.notdead
		move.l	ido(a0),idraw(a0)
		move.l	#dodeadcam,ido(a0)
		move.w	#512,iczoom(a0)
.notdead:
		rts

****************************** Camera Generators *****************************

clearpanningcams:
		clr.w	numpanningcams
		move.l	#panningcams,nextpanningcam
		rts

clearvolumes:
		clr.w	numvolumes
		move.l	#volumes,nextvolume
		rts

******************************** Data Section ********************************

		data

********************************* Variables **********************************

usecam:		dc.b	0
nochangecam:	dc.b	0

***************************** Camera Structures ******************************

		phrase
cameras1:	dc.l	0			;parent
		dc.l	0,0			;previous,next
		dc.l	cam11,cam11		;first,last
		dc.l	dolist,donowt		;do,draw

		phrase
cam11:		dc.l	cameras1		;parent
		dc.l	0,0			;previous,next
		dc.l	0,0			;first,last
		dc.l	donowt,donowt		;do,draw
		dc.l	pad1			;controller
		dc.l	0,0,0			;xcoord,ycoord,zcoord
		dc.w	$4000,0,0		;orientation matrix
		dc.w	0,$4000,0
		dc.w	0,0,$4000
		dc.l	0			;target
		dc.w	camxangle,camyangle,camzangle	;xangle,yangle,zangle
		dc.w	0,0,0			;xrot,yrot,zrot
		dc.w	mincamdist,0		;distance,zoom
cam_xy1:
camwindow:	dc.w	-160,-120		;clx,cly	botleft
		dc.w	160,120			;chx,chy	topright
		dc.w	160,120			;gox,goy	midpoint

		phrase
cameras2:	dc.l	0			;parent
		dc.l	0,0			;previous,next
		dc.l	cam21,cam22		;first,last
		dc.l	dolist,donowt		;do,draw

		phrase
cam21:		dc.l	cameras2		;parent
		dc.l	0,cam22			;previous,next
		dc.l	0,0			;first,last
		dc.l	donowt,donowt		;do,draw
		dc.l	pad1			;controller
		dc.l	0,0,0			;xcoord,ycoord,zcoord
		dc.w	$4000,0,0		;orientation matrix
		dc.w	0,$4000,0
		dc.w	0,0,$4000
		dc.l	0			;target
		dc.w	camxangle,camyangle,camzangle	;xangle,yangle,zangle
		dc.w	0,0,0			;xrot,yrot,zrot
		dc.w	mincamdist,0		;distance,zoom
cam_xy2:	dc.w	-160,-60		;clx,cly
		dc.w	160,60			;chx,chy
		dc.w	160,180			;gox,goy

		phrase
cam22:		dc.l	cameras2		;parent
		dc.l	cam21,0			;previous,next
		dc.l	0,0			;first,last
		dc.l	donowt,donowt		;do,draw
		dc.l	pad2			;controller
		dc.l	0,0,0			;xcoord,ycoord,zcoord
		dc.w	$4000,0,0		;orientation matrix
		dc.w	0,$4000,0
		dc.w	0,0,$4000
		dc.l	0			;target
		dc.w	camxangle,camyangle,camzangle	;xangle,yangle,zangle
		dc.w	0,0,0			;xrot,yrot,zrot
		dc.w	mincamdist,0		;distance,zoom
cam_xy3:	dc.w	-160,-60		;clx,cly
		dc.w	160,60			;chx,chy
		dc.w	160,60			;gox,goy

********************************* BSS Section ********************************

		bss

********************************** Variables *********************************

		even

nextpanningcam:	ds.l	1
nextvolume:	ds.l	1

numpanningcams:	ds.w	1
numvolumes:	ds.w	1

panningcams:	ds.l	12*maxpanningcams
volumes:	ds.l	24*maxvolumes
