******************** Calculate Gouraud Vectors For Shapes ********************

* Notes:
* Put all shapes that need gouraud vectors calculating
* in a table called 'shadetable'
* Then call 'shadeshapes'
* Hey Presto! The shapes are shaded!

* We need to calculate the gouraud vector for each point
* This vector is the average of the surface normal vectors
* for all polygons that contain the point in question

* So what we do is:
* For each point examine the polygons and if any polygons contain
* our point, calculate the surface normal and add it to the running
* total. When there are no more polygons, divide total by number
* of vectors we added, to get the average, then make that vector
* into a unit vector. This is our gouraud vector.

* This should only be necessary for hand coded shapes
* Marvin's converter does this anyway for 3ds shapes

****************************** Code Section **********************************

		text
		.68000

***************************** Shading Shapes *********************************

shadeshapes:	move.l	shade_list_pointer,a0
.shadeshapes:
		move.l	(a0)+,a1
		move.l	a1,d0
		beq	.doneshade
		addq.l	#2,a1
		move.w	(a1)+,d7		;number of points to shade
		move.l	a1,a2			;pointer to points
		move.l	a1,a3
		move.w	d7,d0			;calc pointer to polygons
		lsl.w	#2,d0
		add.w	d0,a3
		add.w	d0,d0
		add.w	d0,a3

		clr.l	gouraudx		;our gouraud vector
		clr.l	gouraudy
		clr.l	gouraudz
		clr.w	curpoint		;current point
		clr.w	numvectors		;number of vectors added
		subq.w	#1,d7
.shadepoints:
		move.l	a3,a4			;step through all the
		move.w	(a4)+,d6		;polygons
		subq.w	#1,d6
.checkpolygons:
		move.l	a4,a5			;step through each polygon
		move.l	(a5)+,d5		;numpoints in low word
		move.w	curpoint,d4
		subq.w	#1,d5
.checkpoly:
		cmp.w	(a5)+,d4		;is the current point
		bne	.no			;in this polygon?

		movem.l	d5-d7/a2-a4,-(sp)	;make some room...

		move.w	4(a4),d0		;yes it is so
		move.w	6(a4),d1		;calculate surface normal
		move.w	8(a4),d2
		move.l	a1,a2
		move.l	a1,a3
		move.l	a1,a4
		lsl.w	#2,d0
		lsl.w	#2,d1
		lsl.w	#2,d2
		add.w	d0,a2
		add.w	d1,a3
		add.w	d2,a4
		add.w	d0,d0
		add.w	d1,d1
		add.w	d2,d2
		add.w	d0,a2
		add.w	d1,a3
		add.w	d2,a4

		movem.w	(a3),d0-d2		;get vectors from point
		movem.w	(a4),d3-d5		;0 to point 1 and from
		sub.w	d0,d3			;point 1 to point 2
		sub.w	d1,d4
		sub.w	d2,d5
		sub.w	(a2)+,d0
		sub.w	(a2)+,d1
		sub.w	(a2)+,d2

		move.w	d1,d6			;calculate normal x
		move.w	d4,d7
		muls	d5,d6
		muls	d2,d7
		sub.l	d7,d6
		move.l	d6,normalx

		move.w	d2,d6			;calculate normal y
		move.w	d5,d7
		muls	d3,d6
		muls	d0,d7
		sub.l	d7,d6
		move.l	d6,normaly

		move.w	d0,d6			;calculate normal z
		move.w	d3,d7
		muls	d4,d6
		muls	d1,d7
		sub.l	d7,d6
		move.l	d6,normalz

		movem.l	normalx,d0-d2		;make normal into a unit vector
		bsr	makeunitvec		;and add to gouraud total
		ext.l	d0
		ext.l	d1
		ext.l	d2
		add.l	d0,gouraudx
		add.l	d1,gouraudy
		add.l	d2,gouraudz

		addq.w	#1,numvectors

		movem.l	(sp)+,d5-d7/a2-a4

		bra.s	.nextpoly
.no:
		dbf	d5,.checkpoly
.nextpoly:
		move.l	(a4)+,d0		;numpoints in low word

		add.w	d0,a4
		add.w	d0,a4
		dbf	d6,.checkpolygons

		move.w	numvectors,d3
		beq.s	.donepoint		;just in case!
		movem.l	gouraudx,d0-d2
		divs	d3,d0
		divs	d3,d1
		divs	d3,d2
		ext.l	d0
		ext.l	d1
		ext.l	d2
		bsr	makeunitvec
		movem.w	d0-d2,6(a2)
.donepoint:
		lea	12(a2),a2		;move onto next point
		addq.w	#1,curpoint
		dbf	d7,.shadepoints
		bra	.shadeshapes		;move onto next shape
.doneshade:
		rts

********************************* BSS Section ********************************

		bss

********************************** Variables *********************************

		even

shade_list_pointer:
		ds.l	1

normalx:	ds.l	1
normaly:	ds.l	1
normalz:	ds.l	1

gouraudx:	ds.l	1
gouraudy:	ds.l	1
gouraudz:	ds.l	1

numvectors:	ds.w	1
curpoint:	ds.w	1

