**************************** Guard Control Routines **************************

* Notes:

* The guard is complicated by having separate head and body objects
* but it's still manageable

* This is the kind of situation where a hierarchial system is useful
* but usually they're more bother than they're worth

********************************* Code Section *******************************

		text
		.68000

****************** Guard Waiting And Scanning For Targets ********************

doguardwait:	tst.w	isonscreen(a0)		;only update when onscreen
		beq.s	.done

		bsr	guardcollide		;deal with collisions
		tst.w	ioenergy(a0)
		ble	explodeguard

		move.l	inext(a0),a1		;set head position
		movem.l	isxcoord(a0),d0-d2	;over the body
		movem.l	d0-d2,isxcoord(a1)

		bsr	readswitch
		beq.s	.done

		subq.w	#1,iotimer(a0)		;look for a target
		bne.s	.done
		move.w	#longaidelay,iotimer(a0)
		bsr	rangetarget
		move.l	a1,iotarget(a0)
		beq.s	.done

		move.l	#doguardattack,ido(a0)	;we have a target!
.done:
		rts

************************* Guard Attacking It's Target ************************

doguardattack:	tst.w	isonscreen(a0)		;only update when onscreen
		beq.s	.done

		bsr	doobjcoll		;deal with collisions
		bsr.s	guardcollide
		tst.w	ioenergy(a0)
		ble	explodeguard

		bsr	confirmtarget
		beq.s	.losttarget
		bsr	flatrotatetotarget
		bsr	flatmoveobject
		bsr	flatadjustspeed

		move.l	inext(a0),a1		;set head position
		movem.l	isxcoord(a0),d0-d2	;over the body
		movem.l	d0-d2,isxcoord(a1)
		
		move.l	a0,-(sp)
		move.l	iotarget(a0),iotarget(a1)
		move.l	a1,a0
		bsr	rotatetotarget
		move.l	(sp)+,a0

		subq.w	#1,iotimer2(a0)		;shoot every now and then
		bne.s	.done
		jsr	random
		and.w	#3,d0
		add.w	#12,d0
		move.w	d0,iotimer2(a0)
		move.l	a0,-(sp)		;shoot from the head
		move.l	inext(a0),a0		;not from the body
		bsr	createenemyfrontlaser1
		move.l	(sp)+,a0
.done:
		rts
.losttarget:
		move.l	#doguardwait,ido(a0)
		rts

****************************** Guard Subroutines *****************************

guardcollide:	move.l	a0,a1
		bsr.s	.partcollide
		move.l	inext(a0),a1
.partcollide:
		tst.l	iocollcode(a1)
		beq.s	.nocollide
		clr.l	iocollcode(a1)
		move.l	iocollobj(a1),a2
		move.w	iodamage(a2),d0
		sub.w	d0,ioenergy(a0)
.nocollide:
		rts

explodeguard:	move.l	inext(a0),a1
		move.l	#dobigbang,ido(a0)
		move.l	#dobigbang,ido(a1)
		addq.w	#1,baddieskilled
		rts

****************************** Object Generators *****************************

makeguard:	bsr	getempty		;make mechanoid guard
		bsr	getempty

		movem.l	position,d0-d2
		add.l	#500,d1
		movem.l	d0-d2,isxcoord(a1)

		movem.w	orientation,d0-d7/a2
		movem.w	d0-d7/a2,isxx(a1)

		move.w	switch,ioswitch(a1)

		movem.l	boundary,d0-d5
		movem.l	d0-d5,iominx(a1)
		move.l	#longrange,iorange(a1)	;set attack range

		move.l	#doguardwait,ido(a1)
		move.l	#guardbodyanim,isanimbase(a1)
		move.l	#0,isanimframe(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#guardTYPE,iotype(a1)
		move.w	#guardENERGY,ioenergy(a1)
		move.w	#guardDAMAGE,iodamage(a1)

		jsr	random			;this is so they don't
		and.w	#31,d0			;all look for targets
		addq.w	#1,d0			;on the same update
		move.w	d0,iotimer(a1)

		move.w	#1,iotimer2(a1)		;shooting delay

		move.l	inext(a1),a2

		movem.l	isxcoord(a1),d0-d2
		movem.l	d0-d2,isxcoord(a2)
		movem.w	isxx(a1),d0-d7/a3
		movem.w	d0-d7/a3,isxx(a2)

		move.l	#donowt,ido(a2)
		move.l	#guardheadanim,isanimbase(a2)
		move.l	#0,isanimframe(a2)

		move.w	#enemyCLASS,ioclass(a2)
		move.w	#guardTYPE,iotype(a2)
		move.w	#guardENERGY,ioenergy(a2)

		move.w	iotimer(a1),iotimer(a2)
		move.w	iotimer2(a1),iotimer2(a2)

		move.w	#0,iobarrelx(a2)
		move.w	#0,iobarrely(a2)
		move.w	#300,iobarrelz(a2)

		addq.w	#1,baddies
		rts

******************************** Data Section ********************************

		data

********************************* Variables **********************************

********************************* Animations *********************************

		phrase
guardheadanim:	dc.l	guardheadvdf

guardbodyanim:	dc.l	guardbodyvdf

********************************* VDF Files **********************************

guardheadvdf:	dc.w	guardheadc-guardheadvdf

guardheads:	dc.w	330

		dc.w	7

		dc.w	-200,0,-200,0,0,0		;0
		dc.w	200,0,-200,0,0,0		;1
		dc.w	-50,50,-50,0,0,0		;2
		dc.w	50,50,-50,0,0,0			;3
		dc.w	-50,0,-50,0,0,0			;4
		dc.w	50,0,-50,0,0,0			;5
		dc.w	0,0,250,0,0,0			;6

		dc.w	9

		dc.w	$8005,3,1,6,3	;3,6,1
		dc.w	$8006,3,3,6,2	;2,6,3
		dc.w	$8007,3,2,6,0	;0,6,2
		dc.w	$8008,3,4,6,5	;5,6,4
		dc.w	$8009,3,0,6,4	;4,6,0
		dc.w	$800a,3,5,6,1	;1,6,5

		dc.w	$4100,3,3,5,1	;1,5,3
		dc.w	$8010,4,4,5,3,2	;2,3,5,4
		dc.w	$4100,3,4,2,0	;0,2,4

guardheadc:	dc.w	330


		dc.w	1

		dc.w	-200,0,-50
		dc.w	400,50,300
		dc.w	static+guardCODE

guardbodyvdf:	dc.w	guardbodyc-guardbodyvdf

guardbodys:	dc.w	550

		dc.w	9

		dc.w	-70,-500,0,0,0,0		;0
		dc.w	70,-500,0,0,0,0			;1
		dc.w	-100,-150,-100,0,0,0		;2
		dc.w	100,-150,-100,0,0,0		;3
		dc.w	-100,-150,100,0,0,0		;4
		dc.w	100,-150,100,0,0,0		;5
		dc.w	-200,-100,0,0,0,0		;6
		dc.w	200,-100,0,0,0,0		;7
		dc.w	0,0,0,0,0,0			;8

		dc.w	12

		dc.w	$4034,3,2,6,0	;0,6,2
		dc.w	$8010,4,1,0,4,5	;5,4,0,1
		dc.w	$4034,3,7,3,1	;1,3,7
		dc.w	$4035,3,5,7,1	;1,7,5
		dc.w	$8010,4,0,1,3,2	;2,3,1,0
		dc.w	$4035,3,6,4,0	;0,4,6

		dc.w	$4036,3,6,2,8	;8,2,6
		dc.w	$4036,3,5,4,8	;8,4,5
		dc.w	$4036,3,3,7,8	;8,7,3
		dc.w	$4037,3,4,6,8	;8,6,4
		dc.w	$4037,3,2,3,8	;8,3,2
		dc.w	$4037,3,7,5,8	;8,5,7

guardbodyc:	dc.w	550

		dc.w	1

		dc.w	-200,-500,-100
		dc.w	400,500,200
		dc.w	solid+guardCODE

