*************************** Map Generation Routines **************************

* Notes:

* All the rooms are generated on a grid
* When the player isn't in a room that room and all objects in it will be too
* far away to be drawn. This is flagged, so that the object update routines
* don't bother updating. It's much simpler than trying to maintain
* loads of seperate object lists, and no slower either.

******************************** Code Section ********************************

		text
		.68000


******************************* Scene Generator ******************************

makeworld:	bsr	clearpanningcams
		bsr	clearvolumes
		bsr	clearswitches
		bsr	clearworld

		clr.w	rooms
		clr.w	baddies
		clr.w	databases
		clr.w	datapods
		clr.w	dataports

		clr.w	maxrooms
		clr.w	maxdatabases
		clr.w	maxdatapods

		clr.w	baddieskilled

		sf.b	exit_on

		move.l	#(-roomspacing)*4,d0	;set first position for room grid
		move.l	d0,d1
		move.l	d0,d2
		movem.l	d0-d2,roomx

		lea	targettable,a0		;clear the target table
		clr.l	(a0)
		move.l	a0,nexttarget

		move.l	arenaanims,a0
makerooms:
		tst.l	(a0)
		beq	.maderooms
		move.l	a0,roomanim
		addq.l	#4,a0

		movem.l	d0-d7/a0-a6,-(sp)

		bsr	getempty		;make an arena room object

		movem.l	roomx,d0-d2		;make sure all the rooms
		add.l	#roomspacing,d0		;are well spaced out
		cmp.l	#roomspacing*4,d0
		ble.s	.ok
		move.l	#(-roomspacing)*4,d0

		add.l	#roomspacing,d1
		cmp.l	#roomspacing*4,d1
		ble.s	.ok
		move.l	#(-roomspacing)*4,d1

		add.l	#roomspacing,d2
		cmp.l	#roomspacing*4,d2
		ble.s	.ok
		move.l	#(-roomspacing)*4,d2
.ok:
		movem.l	d0-d2,roomx
		movem.l	d0-d2,isxcoord(a1)

		movem.w	frontmat,d0-d7/a2
		movem.w	d0-d7/a2,isxx(a1)

		move.l	#roomspacing,iorange(a1)

		move.w	rooms,ioswitch(a1)	;set the room number

		addq.w	#1,rooms
		addq.w	#1,maxrooms

		move.l	#donowt,ido(a1)
		move.l	roomanim,isanimbase(a1)
		move.l	#0,isanimframe(a1)

		move.w	#arenaCLASS,ioclass(a1)
		move.w	#roomTYPE,iotype(a1)
		move.w	#roomDAMAGE,iodamage(a1)

		move.l	isanimbase(a1),a0	;generate map info for that room
		move.l	(a0),a0
		bsr	makemapinfo

		movem.l	(sp)+,d0-d7/a0-a6

		bra	makerooms
.maderooms:
		rts

****************************** Generate Map Info *****************************

makemapinfo:	bsr	sortoutboxes
		bsr	makevolumes
		bsr	makecameras
		bsr	makeobjects
		rts

sortoutboxes:	add.w	(a0),a0			;strip all the different kinds
		addq.l	#2,a0			;of map boxes out of a room's vdf
		move.l	a0,a1
		move.w	(a0)+,d7
		subq.w	#1,d7

		move.l	#objectboxes,nextobjectbox
		move.l	#volumeboxes,nextvolumebox
		move.l	#cameraboxes,nextcamerabox
		move.l	a0,nextcollbox
		move.l	#facingboxes,nextfacingbox
		move.l	#switchboxes,nextswitchbox
		move.l	#destboxes,nextdestbox
		move.l	#indexboxes,nextindexbox

		clr.w	numobjectboxes
		clr.w	numvolumeboxes
		clr.w	numcameraboxes
		clr.w	numcollboxes
		clr.w	numfacingboxes
		clr.w	numswitchboxes
		clr.w	numdestboxes
		clr.w	numindexboxes
.checkboxes:
		move.w	d7,-(sp)

		movem.w	(a0)+,d0-d6		;get collision box

		cmp.w	#$ffff,d6		;remove unwanted collision boxes
		beq	.nextbox

		move.w	d6,d7			;get box type
		rol.w	#3,d7
		and.w	#7,d7

		cmp.w	#collisionbox,d7	;which type is it?
		beq	.acollisionbox
		cmp.w	#volumebox,d7
		beq	.avolumebox
		cmp.w	#panningcambox,d7
		beq	.acamerabox
		cmp.w	#facingbox,d7
		beq.s	.afacingbox
		cmp.w	#switchbox,d7
		beq.s	.aswitchbox
		cmp.w	#destinationbox,d7
		beq.s	.adestbox
		cmp.w	#indexbox,d7
		beq.s	.anindexbox

.anobjectbox:
		move.l	nextobjectbox,a6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextobjectbox
		addq.w	#1,numobjectboxes
		bra	.nextbox
.afacingbox:
		move.l	nextfacingbox,a6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextfacingbox
		addq.w	#1,numfacingboxes
		bra	.nextbox
.aswitchbox:
		move.l	nextswitchbox,a6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextswitchbox
		addq.w	#1,numswitchboxes
		bra	.nextbox
.adestbox:
		move.l	nextdestbox,a6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextdestbox
		addq.w	#1,numdestboxes
		bra.s	.nextbox
.anindexbox:
		move.l	nextindexbox,a6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextindexbox
		addq.w	#1,numindexboxes
		bra.s	.nextbox
.acollisionbox:
		move.l	nextcollbox,a6		;save a collision box
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextcollbox
		addq.w	#1,numcollboxes
		bra.s	.nextbox
.avolumebox:
		move.l	nextvolumebox,a6	;save a volume box
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextvolumebox
		addq.w	#1,numvolumeboxes
		bra.s	.nextbox
.acamerabox:
		move.l	nextcamerabox,a6	;save a camera box
		movem.w	d0-d6,(a6)
		lea	14(a6),a6
		move.l	a6,nextcamerabox
		addq.w	#1,numcameraboxes
.nextbox:
		move.w	(sp)+,d7
		dbf	d7,.checkboxes
		move.w	numcollboxes,(a1)
		rts

makevolumes:	lea	volumeboxes,a0		;make all the volumes
		move.w	numvolumeboxes,d7
		beq	.novolumes
		subq.w	#1,d7
.volumes:
		movem.w	(a0)+,d0-d6
		add.l	d0,d3
		add.l	d1,d4
		add.l	d2,d5

		move.l	#camsize,d6
		add.l	d6,d0
		add.l	d6,d1
		add.l	d6,d2
		sub.l	d6,d3
		sub.l	d6,d4
		sub.l	d6,d5

		add.l	roomx,d0
		add.l	roomy,d1
		add.l	roomz,d2
		add.l	roomx,d3
		add.l	roomy,d4
		add.l	roomz,d5

		move.l	nextvolume,a1
		movem.l	d0-d5,(a1)
		lea	24(a1),a1
		move.l	a1,nextvolume
		addq.w	#1,numvolumes

		cmp.w	#maxvolumes,numvolumes
		ble.s	.nextbox
		illegal
		dc.b	"Too Many Camera Volumes!!"
		even
.nextbox:
		dbf	d7,.volumes
.novolumes:
		rts

makecameras:	lea	cameraboxes,a0		;make all the cameras
		move.w	numcameraboxes,d7
		beq.s	.nocameras
		subq.w	#1,d7
.cameras:
		movem.w	(a0)+,d0-d6
		asr.l	d3
		asr.l	d4
		asr.l	d5
		add.l	d0,d3
		add.l	d1,d4
		add.l	d2,d5

		add.l	roomx,d0
		add.l	roomy,d1
		add.l	roomz,d2

		move.l	nextpanningcam,a6
		movem.l	d0-d2,(a6)
		lea	12(a6),a6
		move.l	a6,nextpanningcam
		addq.w	#1,numpanningcams

		cmp.w	#maxpanningcams,numpanningcams
		ble.s	.nextbox
		illegal
		dc.b	"Too Many Panning Cameras!!"
		even
.nextbox:
		dbf	d7,.cameras
.nocameras:
		rts

makeobjects:	lea	objectboxes,a0		;make all the objects
		move.w	numobjectboxes,d7
		beq	.noobjects
		subq.w	#1,d7
.objects:
		movem.l	d0-d7/a0-a6,-(sp)

		movem.w	(a0),d0-d6		;get that object

		movem.w	d0-d5,findlx
		add.w	d0,findhx
		add.w	d1,findhy
		add.w	d2,findhz

		add.l	roomx,d0
		add.l	roomy,d1
		add.l	roomz,d2

		and.w	#collidemask,d6		;get make routine
		add.w	d6,d6
		add.w	d6,d6
		lea	generatortable,a1
		move.l	0(a1,d6.w),-(sp)

		add.l	d0,d3			;get boundary
		add.l	d1,d4
		add.l	d2,d5
		movem.l	d0-d5,boundary

		add.l	d3,d0			;get position
		add.l	d5,d2
		asr.l	d0
		asr.l	d2
		movem.l	d0-d2,position

		bsr	finddirection		;get orientation ( if any )
		bsr	findswitch		;get switch ( if any )
		bsr	finddestination		;get destination ( if any )
		bsr	findindex		;get index ( if any )

		move.l	(sp)+,a1		;make that object!!
		jsr	(a1)
.nextbox:
		movem.l	(sp)+,d0-d7/a0-a6
		lea	14(a0),a0
		dbf	d7,.objects
.noobjects:
		rts

finddirection:	movem.l	d0-d7/a0-a6,-(sp)	;find a facing box
		lea	facingboxes,a6		;inside the find box
		move.w	numfacingboxes,d7
		beq	.default
		subq.w	#1,d7
.findone:
		movem.w	(a6)+,d0-d6		;get the current facing box
		add.w	d0,d3
		add.w	d1,d4
		add.w	d2,d5

		cmp.w	findlx,d0		;is it inside the find box?
		blt	.next
		cmp.w	findly,d1
		blt.s	.next
		cmp.w	findlz,d2
		blt.s	.next
		cmp.w	findhx,d3
		bgt.s	.next
		cmp.w	findhy,d4
		bgt.s	.next
		cmp.w	findhz,d5
		bgt.s	.next

		add.w	d3,d0			;it is so calculate
		add.w	d4,d1			;an orientation
		add.w	d5,d2
		asr.w	d0
		asr.w	d1
		asr.w	d2
		ext.l	d0
		ext.l	d1
		ext.l	d2
		add.l	roomx,d0
		add.l	roomy,d1
		add.l	roomz,d2
		movem.l	position,d3-d5

		move.l	a6,-(sp)
		lea	orientation,a6
		bsr	flatpointmatrix
		move.l	(sp)+,a6

		move.l	nextfacingbox,a5	;replace that facing box
		lea	-14(a5),a5		;with the last facing box
		lea	-14(a6),a6		;so that we have fewer to
		movem.w	(a5),d0-d6		;search through next time
		movem.w	d0-d6,(a6)
		move.l	a5,nextfacingbox
		subq.w	#1,numfacingboxes

		bra.s	.gotmat
.next:
		dbf	d7,.findone
.default:
		movem.w	frontmat,d0-d7/a1
		movem.w	d0-d7/a1,orientation
.gotmat:
		movem.l	(sp)+,d0-d7/a0-a6
		rts

findindex:	movem.l	d0-d7/a0-a6,-(sp)	;find an index box
		lea	indexboxes,a6		;inside the find box
		move.w	numindexboxes,d7
		beq.s	.default
		subq.w	#1,d7
.findone:
		movem.w	(a6)+,d0-d6		;get the current index box
		add.w	d0,d3
		add.w	d1,d4
		add.w	d2,d5

		cmp.w	findlx,d0		;is it inside the find box?
		blt.s	.next
		cmp.w	findly,d1
		blt.s	.next
		cmp.w	findlz,d2
		blt.s	.next
		cmp.w	findhx,d3
		bgt.s	.next
		cmp.w	findhy,d4
		bgt.s	.next
		cmp.w	findhz,d5
		bgt.s	.next

		and.l	#collidemask,d6		;get the index
		move.l	d6,index

		move.l	nextindexbox,a5		;replace that index box
		lea	-14(a5),a5		;with the last index box
		lea	-14(a6),a6		;so that we have fewer to
		movem.w	(a5),d0-d6		;search through next time
		movem.w	d0-d6,(a6)
		move.l	a5,nextindexbox
		subq.w	#1,numindexboxes

		bra.s	.gotindex
.next:
		dbf	d7,.findone
.default:
		move.l	#0,index
.gotindex:
		movem.l	(sp)+,d0-d7/a0-a6
		rts

findswitch:	movem.l	d0-d7/a0-a6,-(sp)	;find a switch box
		lea	switchboxes,a6		;inside the find box
		move.w	numswitchboxes,d7
		beq.s	.default
		subq.w	#1,d7
.findone:
		movem.w	(a6)+,d0-d6		;get the current switch box
		add.w	d0,d3
		add.w	d1,d4
		add.w	d2,d5

		cmp.w	findlx,d0		;is it inside the find box?
		blt.s	.next
		cmp.w	findly,d1
		blt.s	.next
		cmp.w	findlz,d2
		blt.s	.next
		cmp.w	findhx,d3
		bgt.s	.next
		cmp.w	findhy,d4
		bgt.s	.next
		cmp.w	findhz,d5
		bgt.s	.next

		and.w	#collidemask,d6		;get the switch
		move.w	d6,switch

		move.l	nextswitchbox,a5	;replace that switch box
		lea	-14(a5),a5		;with the last switch box
		lea	-14(a6),a6		;so that we have fewer to
		movem.w	(a5),d0-d6		;search through next time
		movem.w	d0-d6,(a6)
		move.l	a5,nextswitchbox
		subq.w	#1,numswitchboxes

		bra.s	.gotswitch
.next:
		dbf	d7,.findone
.default:
		move.w	#0,switch
.gotswitch:
		movem.l	(sp)+,d0-d7/a0-a6
		rts

finddestination:
		movem.l	d0-d7/a0-a6,-(sp)	;find a destination box
		lea	destboxes,a6		;inside the find box
		move.w	numdestboxes,d7
		beq.s	.default
		subq.w	#1,d7
.findone:
		movem.w	(a6)+,d0-d6		;get the current destination box
		add.w	d0,d3
		add.w	d1,d4
		add.w	d2,d5

		cmp.w	findlx,d0		;is it inside the find box?
		blt.s	.next
		cmp.w	findly,d1
		blt.s	.next
		cmp.w	findlz,d2
		blt.s	.next
		cmp.w	findhx,d3
		bgt.s	.next
		cmp.w	findhy,d4
		bgt.s	.next
		cmp.w	findhz,d5
		bgt.s	.next

		and.w	#collidemask,d6		;get the destination
		move.w	d6,destination

		move.l	nextdestbox,a5		;replace that destination box
		lea	-14(a5),a5		;with the last destination box
		lea	-14(a6),a6		;so that we have fewer to
		movem.w	(a5),d0-d6		;search through next time
		movem.w	d0-d6,(a6)
		move.l	a5,nextdestbox
		subq.w	#1,numdestboxes

		bra.s	.gotdest
.next:
		dbf	d7,.findone
.default:
		move.w	#0,destination
.gotdest:
		movem.l	(sp)+,d0-d7/a0-a6
		rts

******************************** Data Section ********************************

		data

********************************* Variables **********************************

******************************** Generator Table *****************************

generatortable:	dc.l	makeplayer1		;player objects
		dc.l	makeplayer2

		dc.l	makelaser1
		dc.l	makelaser2
		dc.l	makeplasma1
		dc.l	makeplasma2
		dc.l	makeplasma3
		dc.l	makelauncher1
		dc.l	makelauncher2
		dc.l	makelauncher3
		dc.l	makeshields1
		dc.l	makeshields2
		dc.l	makeshields3
		dc.l	makereargun1
		dc.l	makereargun2
		dc.l	makereargun3
		dc.l	makerearmine1
		dc.l	makerearmine2
		dc.l	makerearmine3
		dc.l	makeauto
		dc.l	makeradar1
		dc.l	makeradar2
		dc.l	makedrone1
		dc.l	makedrone2
		dc.l	makelife
		dc.l	makeenergy

		dc.l	makedestination		;arena feature objects
		dc.l	makeexit
		dc.l	makedatapod
		dc.l	makedatabase
		dc.l	makelprism
		dc.l	makerprism
		dc.l	maketimedswitch
		dc.l	maketripswitch
		dc.l	makeswitch
		dc.l	makekey
		dc.l	makedoor
		dc.l	makespring
		dc.l	makerepellor
		dc.l	makeattractor
		dc.l	makeplatform
		dc.l	makewarp
		dc.l	makedistorter
		dc.l	makefloorpanel
		dc.l	makecolumn
		dc.l	maketimedbarrier
		dc.l	makebarrier
		dc.l	makespikes

		dc.l	maketank		;enemy objects
		dc.l	makelauncher
		dc.l	makemobile
		dc.l	makelayer
		dc.l	makegun
		dc.l	makeguard
		dc.l	makebomber
		dc.l	makebouncer
		dc.l	makefragmentor
		dc.l	makephaser
		dc.l	makehomer
		dc.l	maketrundler
		dc.l	makeradar
		dc.l	makedropship
		dc.l	makeresmine

		dc.l	makedataport

********************************* BSS Section ********************************

		bss

********************************** Variables *********************************

		even

roomanim:	ds.l	1

nextobjectbox:	ds.l	1
nextfacingbox:	ds.l	1
nextswitchbox:	ds.l	1
nextdestbox:	ds.l	1
nextindexbox:	ds.l	1
nextvolumebox:	ds.l	1
nextcamerabox:	ds.l	1
nextcollbox:	ds.l	1

numobjectboxes:	ds.w	1
numfacingboxes:	ds.w	1
numswitchboxes:	ds.w	1
numdestboxes:	ds.w	1
numindexboxes:	ds.w	1
numvolumeboxes:	ds.w	1
numcameraboxes:	ds.w	1
numcollboxes:	ds.w	1

objectboxes:	ds.b	14*100		;data for object boxes
facingboxes:	ds.b	14*100		;data for facing boxes
switchboxes:	ds.b	14*100		;data for switch boxes
destboxes:	ds.b	14*100		;data for dest boxes
indexboxes:	ds.b	14*100		;data for index boxes
volumeboxes:	ds.b	14*100		;data for volume boxes
cameraboxes:	ds.b	14*100		;data for camera boxes

findlx:		ds.w	1
findly:		ds.w	1
findlz:		ds.w	1
findhx:		ds.w	1
findhy:		ds.w	1
findhz:		ds.w	1

boundary:	ds.l	6
position:	ds.l	3
orientation:	ds.w	9
switch:		ds.w	1
destination:	ds.w	1
index:		ds.l	1

codes:		ds.w	1000
