******************* Merge A Table Of Shapes Into A New Shape *****************

* Notes:

* Put a pointer to vdf files that you want to merge in a0
* Put a pointer to a buffer for the new vdf file in a1
* Call 'mergeshapes'

* With any luck, you'll get a new merged vdf file

* The vdf files can be put at any position or orientation
* Each vdf file has a single collision box wrapped around it

* The format of the merge table is
*	dc.l	vdf file,x,y,z,xa,ya,za,collision code	(for each vdf file)
*	dc.l	0					(which flags the end)

******************************** Code Section ********************************

		text
		.68000

******************************* Merging Shapes *******************************

mergeshapes:	move.l	a1,newvdf		;save new vdf file pointer

		lea	pointbuffer,a4		;pointer to point buffer
		lea	shadedbuffer,a5		;pointer to shaded polygon buffer
		lea	collidebuffer,a6	;pointer to collision box buffer

		clr.w	pointcount		;zero counts to start with
		clr.w	polygoncount
		clr.w	boxcount
.merge:
		move.w	pointcount,pointssofar	;copy counts so far
		move.w	polygoncount,polygonssofar
		move.w	boxcount,boxessofar

		move.l	(a0)+,d0		;get pointer to next vdf file
		beq	.createvdf		;and quit if no more left
		move.l	d0,a3

		movem.l	(a0)+,d0-d2		;get origin for vdf file
		movem.w	d0-d2,originx

		movem.l	(a0)+,d0-d2		;get orientation for vdf file
		;movem.l	d0-d7/a0-a6,-(sp)
		;lea	mergexx,a6
		;bsr	anglematrix
		;movem.l	(sp)+,d0-d7/a0-a6

		move.l	(a0)+,collisioncode	;save collision code for that vdf file

		addq.l	#4,a3			;skip collision data

		move.w	#32767,d0		;set initial collision box
		move.w	d0,boxminx
		move.w	d0,boxminy
		move.w	d0,boxminz
		neg.w	d0
		move.w	d0,boxmaxx
		move.w	d0,boxmaxy
		move.w	d0,boxmaxz

		move.w	(a3)+,d7		;copy the points
		add.w	d7,pointcount		;to the point buffer
		subq.w	#1,d7
.points:
		movem.w	(a3)+,d0-d2		get rotated coordinate
		;bsr	mergerotate
		add.w	originx,d0
		add.w	originy,d1
		add.w	originz,d2

		cmp.w	boxminx,d0		;check against collision box limits
		bge.s	.notminx
		move.w	d0,boxminx
		bra.s	.notmaxx
.notminx:
		cmp.w	boxmaxx,d0
		ble.s	.notmaxx
		move.w	d0,boxmaxx
.notmaxx:
		cmp.w	boxminy,d1
		bge.s	.notminy
		move.w	d1,boxminy
		bra.s	.notmaxy
.notminy:
		cmp.w	boxmaxy,d1
		ble.s	.notmaxy
		move.w	d1,boxmaxy
.notmaxy:
		cmp.w	boxminz,d2
		bge.s	.notminz
		move.w	d2,boxminz
		bra.s	.notmaxz
.notminz:
		cmp.w	boxmaxz,d2
		ble.s	.notmaxz
		move.w	d2,boxmaxz
.notmaxz:
		movem.w	d0-d2,(a4)
		addq.l	#6,a4

		movem.w	(a3)+,d0-d2		;get rotated gouraud vector
		;bsr	mergerotate
		movem.w	d0-d2,(a4)
		addq.l	#6,a4

		dbf	d7,.points

		move.w	(a3)+,d7		;copy the polygons
		add.w	d7,polygoncount		;to the polygon buffer
		subq.w	#1,d7
.polygons:
		move.w	(a3)+,(a5)+
		move.w	(a3)+,d6
		move.w	d6,(a5)+
		subq.w	#1,d6
.apolygon:
		move.w	(a3)+,d0
		add.w	pointssofar,d0
		move.w	d0,(a5)+
		dbf	d6,.apolygon
		dbf	d7,.polygons

		movem.w	boxminx,d0-d5		;save the collision box
		sub.w	d0,d3			;for that vdf file
		sub.w	d1,d4
		sub.w	d2,d5
		move.l	collisioncode,d6
		movem.w	d0-d6,(a6)
		lea	14(a6),a6

		addq.w	#1,boxcount

		bra	.merge			;go onto the next vdf file
.createvdf:
		lea	pointbuffer,a4		;now create the new vdf file
		lea	shadedbuffer,a5
		lea	collidebuffer,a6

		move.l	newvdf,a1		;get pointer to new vdf data
		addq.l	#4,a1

		move.w	pointcount,d7
		move.w	d7,(a1)+		;number of points
		subq.w	#1,d7
.newpoints:
		movem.w	(a4)+,d0-d5
		movem.w	d0-d5,(a1)		;each point
		lea	12(a1),a1
		dbf	d7,.newpoints

		move.w	polygoncount,d7
		move.w	d7,(a1)+		;number of polygons
		subq.w	#1,d7
.newpolygons:
		move.w	(a5)+,(a1)+		;colour of polygon

		move.w	(a5)+,d6
		move.w	d6,(a1)+		;number of points in polygon
		subq.w	#1,d6
.anewpolygon:
		move.w	(a5)+,(a1)+		;each point in polygon
		dbf	d6,.anewpolygon
		dbf	d7,.newpolygons

		move.l	newvdf,a0		;save offset to collision data
		move.l	a1,d0
		sub.l	a0,d0
		move.w	d0,(a0)

		addq.l	#2,a1
		move.w	boxcount,d7		;copy collision boxes
		move.w	d7,(a1)+
		subq.w	#1,d7
.boxes:
		movem.w	(a6)+,d0-d6
		movem.w	d0-d6,(a1)
		lea	14(a1),a1
		dbf	d7,.boxes

		move.l	newvdf,a1		;calculate radius of new vdf file
		addq.l	#4,a1
		move.w	(a1)+,d7
		subq.w	#1,d7
		moveq.l	#0,d0
.findshaperadius:
		movem.w	(a1),d1-d3
		muls	d1,d1
		muls	d2,d2
		muls	d3,d3
		add.l	d2,d1
		add.l	d3,d1
		cmp.l	d1,d0
		bge.s	.nope
		move.l	d1,d0
.nope:
		lea	12(a1),a1
		dbf	d7,.findshaperadius
		bsr	root

		move.l	newvdf,a1
		move.l	a1,a2
		add.w	(a2),a2
		move.w	d0,2(a1)		;save shape radius
		move.w	d0,(a2)			;save collision radius
		rts

mergerotate:	move.w	mergexx,d3		;calculate rotated xcoord
		move.w	mergeyx,d4
		move.w	mergezx,d5
		muls	d0,d3
		muls	d1,d4
		muls	d2,d5
		add.l	d4,d3
		add.l	d5,d3
		add.l	d3,d3
		add.l	d3,d3
		swap	d3
		move.w	d3,pointx

		move.w	mergexy,d3		;calculate rotated ycoord
		move.w	mergeyy,d4
		move.w	mergezy,d5
		muls	d0,d3
		muls	d1,d4
		muls	d2,d5
		add.l	d4,d3
		add.l	d5,d3
		add.l	d3,d3
		add.l	d3,d3
		swap	d3
		move.w	d3,pointy

		move.w	mergexz,d3		;calculate rotated zcoord
		move.w	mergeyz,d4
		move.w	mergezz,d5
		muls	d0,d3
		muls	d1,d4
		muls	d2,d5
		add.l	d4,d3
		add.l	d5,d3
		add.l	d3,d3
		add.l	d3,d3
		swap	d3
		move.w	d3,pointz

		movem.w	pointx,d0-d2		;done it
		rts

********************************* BSS Section ********************************

		bss

********************************** Variables *********************************

		even

newvdf:		ds.l	1		;pointer to new vdf file buffer
collisioncode:	ds.l	1

pointcount:	ds.w	1		;various counters
polygoncount:	ds.w	1
boxcount:	ds.w	1

pointssofar:	ds.w	1		;more counters
polygonssofar:	ds.w	1
boxessofar:	ds.w	1

originx:	ds.w	1		;origin for vdf file
originy:	ds.w	1
originz:	ds.w	1

mergexx:	ds.w	1		;orientation matrix for vdf file
mergexy:	ds.w	1
mergexz:	ds.w	1
mergeyx:	ds.w	1
mergeyy:	ds.w	1
mergeyz:	ds.w	1
mergezx:	ds.w	1
mergezy:	ds.w	1
mergezz:	ds.w	1

pointx:		ds.w	1		;rotated point
pointy:		ds.w	1
pointz:		ds.w	1

boxminx:	ds.w	1		;collision box low coordinates
boxminy:	ds.w	1
boxminz:	ds.w	1
boxmaxx:	ds.w	1		;collision box high coordinates
boxmaxy:	ds.w	1
boxmaxz:	ds.w	1

*************************** Buffer For Collision Boxes ***********************

collidebuffer:	ds.b	14*10		;up to 10 collision boxes
