**************************** Tank Control Routines ***************************

* Notes:

* This file includes the drop tank routines

********************************* Code Section *******************************

		text
		.68000

**************** Tank Falling To The Floor After Being Dropped ***************

dotankfall:	tst.w	isonscreen(a0)		;only update when onscreen
		beq	.done

		bsr	doobjcoll
		bsr	tankcollide
		tst.w	ioenergy(a0)
		ble	explodetank

		move.l	iocollfloor(a0),d1	;invert vertical speed
		beq.s	.yok			;if we hit the floor
		clr.l	iocollfloor(a0)
		add.l	d1,isycoord(a0)
		neg.w	ioyspeed(a0)
		asr.w	ioyspeed(a0)
		cmp.w	#15,ioyspeed(a0)
		bgt.s	.yok
		move.l	#dotankwait,ido(a0)
		clr.w	ioxspeed(a0)
		clr.w	ioyspeed(a0)
		clr.w	iozspeed(a0)
		bra.s	.done
.yok:
		movem.w	ioxspeed(a0),d0-d2 	;move the tank and
		move.w	d0,d3			;keep within boundaries
		move.w	d2,d4
		asr.w	#3,d3
		asr.w	#3,d4
		sub.w	d3,d0
		sub.w	d4,d2
		sub.w	#gravity,d1
		cmp.w	#-terminalv,d1
		bge.s	.ysok
		move.w	#-terminalv,d1
.ysok:
		movem.w	d0-d2,ioxspeed(a0)
		ext.l	d0
		ext.l	d1
		ext.l	d2
		add.l	isxcoord(a0),d0
		add.l	isycoord(a0),d1
		add.l	iszcoord(a0),d2

		cmp.l	iominx(a0),d0
		bge.s	.notminx
		move.l	iominx(a0),d0
		bra.s	.notmaxx
.notminx:
		cmp.l	iomaxx(a0),d0
		ble.s	.notmaxx
		move.l	iomaxx(a0),d0
.notmaxx:
		cmp.l	iominz(a0),d2
		bge.s	.notminz
		move.l	iominz(a0),d2
		bra.s	.notmaxz
.notminz:
		cmp.l	iomaxz(a0),d2
		ble.s	.notmaxz
		move.l	iomaxz(a0),d2
.notmaxz:
		movem.l	d0-d2,isxcoord(a0)
.done:
		rts

****************** Tank Waiting And Scanning For Targets *********************

dotankwait:	tst.w	isonscreen(a0)		;only update when onscreen
		beq.s	.done

		bsr	tankcollide		;deal with collisions
		tst.w	ioenergy(a0)
	 	ble	explodetank

		bsr	readswitch		;wait for switch on
		beq.s	.done

		subq.w	#1,iotimer(a0)		;look for a target
		bne.s	.done
		move.w	#longaidelay,iotimer(a0)
		bsr	rangetarget
		move.l	a1,iotarget(a0)
		beq.s	.done

		move.l	#dotankattack,ido(a0)	;we have a target!
.done:
		rts

************************* Tank Attacking It's Target *************************

dotankattack:	tst.w	isonscreen(a0)		;only update when onscreen
		beq.s	.done

		bsr	doobjcoll		;deal with collisions
		bsr.s	tankcollide
		tst.w	ioenergy(a0)
	 	ble.s	explodetank

		bsr	confirmtarget
		beq.s	.losttarget
		bsr	flatrotatetotarget
	 	bsr	flatmoveobject
		bsr	flatadjustspeed

		subq.w	#1,iotimer2(a0)		;shoot every now and then
		bne.s	.done
		jsr	random
		and.w	#3,d0
		add.w	#12,d0
		move.w	d0,iotimer2(a0)
		bsr	createenemyfrontlaser1
		neg.w	iobarrelx(a0)		;swap barrels
.done:
		rts
.losttarget:
		move.l	#dotankwait,ido(a0)
		rts

****************************** Tank Subroutines ******************************

tankcollide:	move.l	iocollcode(a0),d0
		beq.s	.nocollide
		clr.l	iocollcode(a0)

		move.l	iocollobj(a0),a1
		move.w	iodamage(a1),d0
		sub.w	d0,ioenergy(a0)
.nocollide:
		rts

explodetank:	cmp.w	#droptankTYPE,iotype(a0)
		bne.s	.notdroptank
		subq.w	#1,enemiesdropped
.notdroptank:
		move.l	#dobigbang,ido(a0)
		addq.w	#1,baddieskilled
		rts

****************************** Object Generators *****************************

createdroptank:	move.w	enemiesavailable,d0    	;make droptank object
		cmp.w	enemiesdropped,d0
		beq	.done
		bsr	getempty
		beq	.done
		addq.w	#1,enemiesdropped

		move.l	#dotankfall,ido(a1)
		move.l	#droptankanim,isanimbase(a1)
		move.l	#0,isanimframe(a1)

		movem.w	isxx(a0),d0-d7/a2
		movem.w	d0-d7/a2,isxx(a1)

		moveq.l	#0,d0
		bsr	movetobarrel

		movem.l	iominx(a0),d0-d5	;give the drop tank the same
		movem.l	d0-d5,iominx(a1)	;boundaries as the drop ship

		move.w	d6,d0			;inherits the speed
		move.w	a2,d1			;of the parent
		move.w	iozspeed(a0),d7
		muls	d7,d0
		muls	d7,d1
		add.l	d0,d0
		add.l	d0,d0
		add.l	d1,d1
		add.l	d1,d1
		swap	d0
		swap	d1
		move.w	d0,ioxspeed(a1)
		clr.w	ioyspeed(a1)
		move.w	d1,iozspeed(a1)

		move.l	#midrange,iorange(a1)	;set attack range

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#droptankTYPE,iotype(a1)
		move.w	#droptankENERGY,ioenergy(a1)
		move.w	#droptankDAMAGE,iodamage(a1)

		move.w	#100,iobarrelx(a1)
		move.w	#100,iobarrely(a1)
		move.w	#300,iobarrelz(a1)

		jsr	random			;delay for ai
		and.w	#31,d0
		addq.w	#1,d0
		move.w	d0,iotimer(a1)

		move.w	#1,iotimer2(a1)		;shooting delay

		clr.w	ioswitch(a1)		;make sure it's active
		addq.w	#1,baddies
.done:
		rts

maketank:	bsr	getempty		;make tank object

		movem.l	position,d0-d2
		movem.l	d0-d2,isxcoord(a1)

		movem.w	orientation,d0-d7/a2
		movem.w	d0-d7/a2,isxx(a1)

		movem.l	boundary,d0-d5
		movem.l	d0-d5,iominx(a1)

		move.l	#dotankwait,ido(a1)
		move.l	#0,isanimframe(a1)

		move.l	#midrange,iorange(a1)	;set attack range

		jsr	random			;delay for ai
		and.w	#3,d0
		addq.w	#1,d0
		move.w	d0,iotimer(a1)

		move.w	switch,ioswitch(a1)

		move.w	#1,iotimer2(a1)		;shooting delay

		addq.w	#1,baddies

		move.l	index,d0
		cmp.l	#4,d0
		beq	.type5
		cmp.l	#3,d0
		beq	.type4
		cmp.l	#2,d0
		beq	.type3
		cmp.l	#1,d0
		beq	.type2
.type1:
		move.l	tank1height,d0
		add.l	d0,isycoord(a1)
		move.l	#tank1anim,isanimbase(a1)

		move.w	#0,iobarrelx(a1)
		move.w	#0,iobarrely(a1)
		move.w	#400,iobarrelz(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#tank1TYPE,iotype(a1)
		move.w	#tank1ENERGY,ioenergy(a1)
		move.w	#tank1DAMAGE,iodamage(a1)
		rts
.type2:
		move.l	tank2height,d0
		add.l	d0,isycoord(a1)
		move.l	#tank2anim,isanimbase(a1)

		move.w	#225,iobarrelx(a1)
		move.w	#-25,iobarrely(a1)
		move.w	#500,iobarrelz(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#tank2TYPE,iotype(a1)
		move.w	#tank2ENERGY,ioenergy(a1)
		move.w	#tank2DAMAGE,iodamage(a1)
		rts
.type3:
		move.l	tank3height,d0
		add.l	d0,isycoord(a1)
		move.l	#tank3anim,isanimbase(a1)

		move.w	#50,iobarrelx(a1)
		move.w	#175,iobarrely(a1)
		move.w	#250,iobarrelz(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#tank3TYPE,iotype(a1)
		move.w	#tank3ENERGY,ioenergy(a1)
		move.w	#tank3DAMAGE,iodamage(a1)
		rts
.type4:
		move.l	tank4height,d0
		add.l	d0,isycoord(a1)
		move.l	#tank4anim,isanimbase(a1)

		move.w	#0,iobarrelx(a1)
		move.w	#75,iobarrely(a1)
		move.w	#350,iobarrelz(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#tank4TYPE,iotype(a1)
		move.w	#tank4ENERGY,ioenergy(a1)
		move.w	#tank4DAMAGE,iodamage(a1)
		rts
.type5:
		move.l	tank5height,d0
		add.l	d0,isycoord(a1)
		move.l	#tank5anim,isanimbase(a1)

		move.w	#150,iobarrelx(a1)
		move.w	#0,iobarrely(a1)
		move.w	#550,iobarrelz(a1)

		move.w	#enemyCLASS,ioclass(a1)
		move.w	#tank5TYPE,iotype(a1)
		move.w	#tank5ENERGY,ioenergy(a1)
		move.w	#tank5DAMAGE,iodamage(a1)
		rts

******************************** Data Section ********************************

		data

********************************* Variables **********************************

********************************* Animations *********************************

		phrase
tank1anim:	dc.l	tank1vdf
tank2anim:	dc.l	tank2vdf
tank3anim:	dc.l	tank3vdf
tank4anim:	dc.l	tank4vdf
tank5anim:	dc.l	tank5vdf

droptankanim:	dc.l	droptankvdf

********************************* VDF Files **********************************

tank1vdf: 	incbin	"graphics\\shapes\\enemies\\etank01.vdf"
tank2vdf: 	incbin	"graphics\\shapes\\enemies\\etank02.vdf"
tank3vdf: 	incbin	"graphics\\shapes\\enemies\\etank03.vdf"
tank4vdf: 	incbin	"graphics\\shapes\\enemies\\etank04.vdf"
tank5vdf: 	incbin	"graphics\\shapes\\enemies\\etank05.vdf"

droptankvdf: 	incbin	"graphics\\shapes\\enemies\\droptank.vdf"

********************************* BSS Section ********************************

		bss

********************************** Variables *********************************

		even

tank1height:	ds.l	1
tank2height:	ds.l	1
tank3height:	ds.l	1
tank4height:	ds.l	1
tank5height:	ds.l	1
