******************** Calculate 3D Collisions With The GPU *****************

* Notes:

* Before use:
* Put a pointer to the shape to check in 'shapeitem'
* Put a pointer to the shape list to check against in 'shapelist'

* Returns a collision code in 'iocollcode'
* and a pointer to the object involved in 'iocollobj'

* It is up to the calling routines to react to the collision
* These routines just DETECT them

* Long variables must precede any words or bytes
* and must be longword aligned

******************************* Code Section ******************************

	text
	.68000

****************** Initialise Object Collision Detection ******************

initobjcoll:	lea	gpuobjcoll,a6		;copy the collision
		bsr	copygpu			;detect routine into
		bsr	waitgpu			;GPU ram
		rts

************************** Detect Object Collisions ***********************

doobjcoll:	move.l	a0,shapeitem		;set shape to detect
		move.l	iparent(a0),shapelist	;set list to detect against

		lea	gpuobjcoll,a6		;do the detection
		bsr	startgpu
		bsr	waitgpu
		rts

************************ GPU Detect Object Collisions *********************

	phrase
objcoll:
	.gpu
	org	gpuprog

	movei	#shapelist,r0			;get object a
	movei	#shapeitem,r1			;and object list
	load	(r0),r0
	load	(r1),r1

	moveq	#ifirst,r2			;get object b
	add	r0,r2
	load	(r2),r0
	movei	#.donelist,r2
	cmpq	#0,r0
	jump	eq,(r2)
	nop
.nextpair:
	movei	#.donepair,r2			;don't detect between
	movei	#iotype,r3			;objects with the same type
	move	r3,r4
	add	r0,r3
	add	r1,r4
	loadw	(r3),r3
	loadw	(r4),r4
	cmp	r3,r4
	jump	eq,(r2)
	nop

	movei	#isblowup,r3			;if either object is
	move	r3,r4				;exploding don't do
	add	r0,r3				;any detection
	add	r1,r4
	loadw	(r3),r3
	loadw	(r4),r4
	shlq	#16,r3
	shlq	#16,r4
	sharq	#16,r3
	sharq	#16,r4
	cmpq	#0,r3
	jump	mi,(r2)
	cmpq	#0,r4
	jump	mi,(r2)

	moveta	r0,r0				;detect object a against
	moveta	r1,r1				;object b and object b
	moveq	#2,r2				;against object a
.bothways:
	moveta	r2,r2
	moveq	#isanimbase,r2			;get collision definitions
	move	r2,r3				;for both objects
	movei	#isanimframe,r4
	move	r4,r5
	add	r0,r2
	add	r1,r3
	add	r0,r4
	add	r1,r5
	load	(r2),r2
	load	(r3),r3

	movei	#.donepair,r9			;if either object has
	cmpq	#0,r2				;no shape defined
	jump	eq,(r9)				;don't do anything
	cmpq	#0,r3
	jump	eq,(r9)
	nop

	load	(r4),r4
	load	(r5),r5
	shlq	#2,r4
	shlq	#2,r5
	add	r4,r2
	add	r5,r3
	load	(r2),r2
	load	(r3),r3
	loadw	(r2),r4				;offset to bounding boxes
	loadw	(r3),r5

	movei	#.donepair,r9			;if either object has
	cmpq	#0,r4				;if no collision boxes
	jump	eq,(r9)
	cmpq	#0,r5
	jump	eq,(r9)

	add	r4,r2
	add	r5,r3

	loadw	(r2),r7				;get collision distances
	loadw	(r3),r8
	addq	#2,r2
	addq	#2,r3
	add	r8,r7				;calculate actual distance

	movei	#isxcoord,r9			;get object b's coords
	move	r9,r10				;relative to object a
	add	r0,r9
	add	r1,r10
	load	(r9),r4
	addq	#4,r9
	load	(r9),r5
	addq	#4,r9
	load	(r9),r6
	load	(r10),r8
	addq	#4,r10
	load	(r10),r9
	addq	#4,r10
	load	(r10),r10
	sub	r8,r4
	sub	r9,r5
	sub	r10,r6

	movei	#.donepair,r8			;do distance reject
	cmp	r7,r4
	jump	pl,(r8)
	cmp	r7,r5
	jump	pl,(r8)
	cmp	r7,r6
	jump	pl,(r8)
	neg	r7
	cmp	r4,r7
	jump	pl,(r8)
	cmp	r5,r7
	jump	pl,(r8)
	cmp	r6,r7
	jump	pl,(r8)
	nop

	movei	#isxx,r15			;get object a's inverse
	add	r1,r15				;orientation matrix
	loadw	(r15),r7
	addq	#2,r15
	loadw	(r15),r10
	addq	#2,r15
	loadw	(r15),r13
	addq	#2,r15
	loadw	(r15),r8
	addq	#2,r15
	loadw	(r15),r11
	addq	#2,r15
	loadw	(r15),r14
	addq	#2,r15
	loadw	(r15),r9
	addq	#2,r15
	loadw	(r15),r12
	addq	#2,r15
	loadw	(r15),r15

	imultn	r4,r7				;rotate object b's coords
	imacn	r5,r10				;relative to object a
	imacn	r6,r13
	resmac	r16
	imultn	r4,r8
	imacn	r5,r11
	imacn	r6,r14
	resmac	r17
	imultn	r4,r9
	imacn	r5,r12
	imacn	r6,r15
	resmac	r18
	move	r16,r4
	move	r17,r5
	move	r18,r6
	sharq	#14,r4
	sharq	#14,r5
	sharq	#14,r6

	movei	#isxx,r24			;get object b's
	add	r0,r24				;orientation matrix
	loadw	(r24),r16
	addq	#2,r24
	loadw	(r24),r17
	addq	#2,r24
	loadw	(r24),r18
	addq	#2,r24
	loadw	(r24),r19
	addq	#2,r24
	loadw	(r24),r20
	addq	#2,r24
	loadw	(r24),r21
	addq	#2,r24
	loadw	(r24),r22
	addq	#2,r24
	loadw	(r24),r23
	addq	#2,r24
	loadw	(r24),r24

	imultn	r7,r16				;get object b's orientation
	imacn	r8,r19				;relative to object a's
	imacn	r9,r22				;kept in alternate registers
	resmac	r25
	sharq	#14,r25
	moveta	r25,r7
	imultn	r7,r17
	imacn	r8,r20
	imacn	r9,r23
	resmac	r25
	sharq	#14,r25
	moveta	r25,r8
	imultn	r7,r18
	imacn	r8,r21
	imacn	r9,r24
	resmac	r25
	sharq	#14,r25
	moveta	r25,r9

	imultn	r10,r16
	imacn	r11,r19
	imacn	r12,r22
	resmac	r25
	sharq	#14,r25
	moveta	r25,r10
	imultn	r10,r17
	imacn	r11,r20
	imacn	r12,r23
	resmac	r25
	sharq	#14,r25
	moveta	r25,r11
	imultn	r10,r18
	imacn	r11,r21
	imacn	r12,r24
	resmac	r25
	sharq	#14,r25
	moveta	r25,r12

	imultn	r13,r16
	imacn	r14,r19
	imacn	r15,r22
	resmac	r25
	sharq	#14,r25
	moveta	r25,r13
	imultn	r13,r17
	imacn	r14,r20
	imacn	r15,r23
	resmac	r25
	sharq	#14,r25
	moveta	r25,r14
	imultn	r13,r18
	imacn	r14,r21
	imacn	r15,r24
	resmac	r25
	sharq	#14,r25
	moveta	r25,r15

	loadw	(r2),r16			;number of boxes
	addq	#2,r2				;in object b

	moveta	r3,r3				;save object a's definition
	moveta	r4,r4				;save object b's origin
	moveta	r5,r5
	moveta	r6,r6
.boxesb:
	loadw	(r2),r18			;get box b's origin
	addq	#2,r2
	loadw	(r2),r19
	addq	#2,r2
	loadw	(r2),r20
	addq	#2,r2

	movefa	r7,r7				;get orientation
	movefa	r8,r8
	movefa	r9,r9
	movefa	r10,r10
	movefa	r11,r11
	movefa	r12,r12
	movefa	r13,r13
	movefa	r14,r14
	movefa	r15,r15

	imultn	r18,r7				;rotate box b's origin
	imacn	r19,r10
	imacn	r20,r13
	resmac	r21
	imultn	r18,r8
	imacn	r19,r11
	imacn	r20,r14
	resmac	r22
	imultn	r18,r9
	imacn	r19,r12
	imacn	r20,r15
	resmac	r23
	sharq	#14,r21
	sharq	#14,r22
	sharq	#14,r23

	add	r21,r4				;add on object b's origin
	add	r22,r5
	add	r23,r6

	loadw	(r2),r18			;get box b's dimensions
	addq	#2,r2
	loadw	(r2),r19
	addq	#2,r2
	loadw	(r2),r20
	addq	#2,r2

	imult	r18,r7				;rotate width
	imult	r18,r8
	imult	r18,r9
	sharq	#14,r7
	sharq	#14,r8
	sharq	#14,r9

	imult	r19,r10				;rotate height
	imult	r19,r11
	imult	r19,r12
	sharq	#14,r10
	sharq	#14,r11
	sharq	#14,r12

	imult	r20,r13				;rotate length
	imult	r20,r14
	imult	r20,r15
	sharq	#14,r13
	sharq	#14,r14
	sharq	#14,r15

	moveta	r4,r16				;save box b's rotated origin
	moveta	r5,r17
	moveta	r6,r18

	loadw	(r3),r17
	addq	#2,r3
.boxesa:
	loadw	(r3),r18			;get box's origin
	addq	#2,r3
	loadw	(r3),r19
	addq	#2,r3
	loadw	(r3),r20
	addq	#2,r3
	loadw	(r3),r21			;get box's dimensions
	addq	#2,r3
	loadw	(r3),r22
	addq	#2,r3
	loadw	(r3),r23
	addq	#2,r3
	loadw	(r3),r24			;get box's collision code

	shlq	#16,r18				;sign extend box's
	shlq	#16,r19				;origin and dimensions
	shlq	#16,r20
	shlq	#16,r21
	shlq	#16,r22
	shlq	#16,r23
	sharq	#16,r18
	sharq	#16,r19
	sharq	#16,r20
	sharq	#16,r21
	sharq	#16,r22
	sharq	#16,r23

	add	r18,r21				;add origin to dimensions
	add	r19,r22
	add	r20,r23

	btst	#floorbit,r24			;check if this box is
	movei	#.boxcollide,r25		;a floor box
	jump	eq,(r25)
	nop
.floorcollide:
	movei	#.nexta,r25			;check whether box b
	cmp	r4,r18				;is inside the floor box
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r23,r6
	jump	pl,(r25)
	nop

	sub	r5,r22				;store how much
	movei	#iocollfloor,r18		;object b is
	add	r0,r18				;inside box a
	store	r22,(r18)

	movei	#isxcoord,r18			;move object b
	movei	#ioxspeed,r19			;by object a's movement
	add	r0,r18
	add	r1,r19
	load	(r18),r20			;get x,z coordinates
	addq	#8,r18
	load	(r18),r21
	loadw	(r19),r22			;get x,z speeds
	addq	#4,r19
	loadw	(r19),r23

	shlq	#16,r22				;sign extend speeds
	shlq	#16,r23
	sharq	#16,r22
	sharq	#16,r23

	add	r22,r20				;add to coords
	add	r23,r21

	store	r21,(r18)			;save coords
	subq	#8,r18
	store	r20,(r18)

	movei	#.nexta,r18
	jump	t,(r18)
	nop

.boxcollide:
	movei	#.dohit,r24
.point1:
	movei	#.point2,r25
	cmp	r4,r18				;check against all points
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point2:
	add	r7,r4				;add width
	add	r8,r5
	add	r9,r6
	movei	#.point3,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point3:
	add	r10,r4				;add height
	add	r11,r5
	add	r12,r6
	movei	#.point4,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point4:
	sub	r7,r4				;subtract width
	sub	r8,r5
	sub	r9,r6
	movei	#.point5,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point5:
	add	r13,r4				;add length
	add	r14,r5
	add	r15,r6
	movei	#.point6,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point6:
	add	r7,r4				;add width
	add	r8,r5
	add	r9,r6
	movei	#.point7,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point7:
	sub	r10,r4				;subtract height
	sub	r11,r5
	sub	r12,r6
	movei	#.point8,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.point8:
	sub	r7,r4				;subtract width
	sub	r8,r5
	sub	r9,r6
	movei	#.nexta,r25
	cmp	r4,r18
	jump	pl,(r25)
	cmp	r5,r19
	jump	pl,(r25)
	cmp	r6,r20
	jump	pl,(r25)
	cmp	r21,r4
	jump	pl,(r25)
	cmp	r22,r5
	jump	pl,(r25)
	cmp	r6,r23
	jump	pl,(r24)
	nop

.nexta:
	addq	#2,r3				;skip collision code

	movefa	r16,r4				;get box b's origin back
	movefa	r17,r5
	movefa	r18,r6

	movei	#.boxesa,r18			;do all the boxes
	subq	#1,r17				;in object a
	jump	ne,(r18)
	nop

.nextb:
	addq	#2,r2				;skip collision code

	movefa	r3,r3				;restore object a's definition
	movefa	r4,r4				;restore object b's origin
	movefa	r5,r5
	movefa	r6,r6

	movei	#.boxesb,r17			;do all the boxes
	subq	#1,r16				;in object b
	jump	ne,(r17)
	nop

.again:
	move	r0,r2				;exchange the pair
	move	r1,r0				;and do it all again
	move	r2,r1
	movei	#.bothways,r3
	movefa	r2,r2
	subq	#1,r2
	jump	ne,(r3)
	nop

	movefa	r0,r0				;go onto the next objects
	movefa	r1,r1
.donepair:
	moveq	#inext,r2
	add	r0,r2
	load	(r2),r0
	movei	#.nextpair,r2
	cmpq	#0,r0
	jump	ne,(r2)
	nop

.donelist:
	movei	#semaphore,r1
	moveq	#0,r0
	moveq	#8,r2				;stop gpu
	movei	#G_CTRL,r3
	store	r0,(r1)
	store	r2,(r3)
.halt:
	jr	t,.halt
	nop

.dohit:
	movei	#.nexta,r31			;deal with the collision

	loadw	(r2),r29			;get the collision codes
	loadw	(r3),r30

	movei	#iocollobj,r25			;save the collision objects
	move	r25,r26
	add	r0,r25
	add	r1,r26
	store	r1,(r25)
	store	r0,(r26)

	movei	#iocollcode-iocollobj,r27	;save the collision codes
	add	r27,r25
	add	r27,r26
	movei	#collidemask,r27
	move	r27,r28
	and	r29,r27
	and	r30,r28
	store	r28,(r25)
	store	r27,(r26)

	btst	#solidbit,r29			;only move the objects
	jump	eq,(r31)			;if they are both solid
	btst	#solidbit,r30
	jump	eq,(r31)
	nop

	movefa	r4,r7				;subroutine to deal
	movefa	r5,r8				;with a collision
	movefa	r6,r9

	add	r21,r18				;get cube a's midpoint
	add	r22,r19
	add	r23,r20
	sharq	#1,r18
	sharq	#1,r19
	sharq	#1,r20
	sub	r18,r21				;find vector to corner
	sub	r19,r22
	sub	r20,r23

	sub	r18,r7				;find vector to object
	sub	r19,r8				;b's origin
	sub	r20,r9

	moveq	#0,r28				;clear direction code

	neg	r23				;check the point against
	imultn	r7,r23				;the two cube planes
	imacn	r9,r21
	resmac	r27
	cmpq	#0,r27
	jr	pl,.above0
	neg	r23
	addq	#1,r28
.above0:
	add	r28,r28
	imultn	r7,r23
	imacn	r9,r21
	resmac	r27
	cmpq	#0,r27
	jr	pl,.above1
	nop
	addq	#1,r28
.above1:

	movei	#.forwards,r27			;decide which cone
	cmpq	#0,r28				;the point is in
	jump	eq,(r27)
	nop

	movei	#.left,r27
	cmpq	#1,r28
	jump	eq,(r27)
	nop

	movei	#.right,r27
	cmpq	#2,r28
	jump	eq,(r27)
	nop

.backwards:
	sub	r23,r20
	sub	r6,r20
	moveq	#0,r7
	moveq	#0,r8
	move	r20,r9
	movei	#.moveit,r28
	jump	t,(r28)
	nop

.forwards:
	add	r23,r20
	sub	r6,r20
	moveq	#0,r7
	moveq	#0,r8
	move	r20,r9
	movei	#.moveit,r28
	jump	t,(r28)
	nop
.left:
	sub	r21,r18
	sub	r4,r18
	move	r18,r7
	moveq	#0,r8
	moveq	#0,r9
	movei	#.moveit,r28
	jump	t,(r28)
	nop
.right:
	add	r21,r18
	sub	r4,r18
	move	r18,r7
	moveq	#0,r8
	moveq	#0,r9

.moveit:
	movefa	r4,r18				;modify object b's
	movefa	r5,r19				;rotated origin
	movefa	r6,r20
	add	r7,r18
	add	r8,r19
	add	r9,r20
	moveta	r18,r4
	moveta	r19,r5
	moveta	r20,r6

	movefa	r16,r18				;modify box b's
	movefa	r17,r19				;rotated origin
	movefa	r18,r20
	add	r7,r18
	add	r8,r19
	add	r9,r20
	moveta	r18,r16
	moveta	r19,r17
	moveta	r20,r18

	movei	#isxx,r28			;get object a's
	add	r1,r28				;orientation matrix
	loadw	(r28),r18
	addq	#2,r28
	loadw	(r28),r19
	addq	#2,r28
	loadw	(r28),r20
	addq	#2,r28
	loadw	(r28),r21
	addq	#2,r28
	loadw	(r28),r22
	addq	#2,r28
	loadw	(r28),r23
	addq	#2,r28
	loadw	(r28),r24
	addq	#2,r28
	loadw	(r28),r25
	addq	#2,r28
	loadw	(r28),r26

	imultn	r7,r18				;rotate the vector
	imacn	r8,r21
	imacn	r9,r24
	resmac	r4
	imultn	r7,r19
	imacn	r8,r22
	imacn	r9,r25
	resmac	r5
	imultn	r7,r20
	imacn	r8,r23
	imacn	r9,r26
	resmac	r6
	sharq	#15,r4				;this halves the vector
	sharq	#15,r5				;as well as rescales it
	sharq	#15,r6

	movei	#isxcoord,r20			;get the object's coords
	move	r20,r21
	add	r0,r20
	add	r1,r21
	load	(r20),r22
	load	(r21),r25
	addq	#4,r20
	addq	#4,r21
	load	(r20),r23
	load	(r21),r26
	addq	#4,r20
	addq	#4,r21
	load	(r20),r24
	load	(r21),r27

	btst	#staticbit,r29
	jr	eq,.moveone
	nop
	btst	#staticbit,r30
	jump	ne,(r31)
	nop
	sub	r4,r25				;if object 1 static
	sub	r5,r26				;and object 2 non static
	sub	r6,r27
	movei	#.moved,r28
	jump	t,(r28)
	nop
.moveone:
	btst	#staticbit,r30
	jr	eq,.moveboth
	nop
	add	r4,r22				;if object 1 non static
	add	r5,r23				;and object 2 static
	add	r6,r24
	jr	t,.moved
	nop
.moveboth:
	sharq	#1,r4				;if both objects non static
	sharq	#1,r5				;move both by half vector
	sharq	#1,r6
	add	r4,r22
	add	r5,r23
	add	r6,r24
	sub	r4,r25
	sub	r5,r26
	sub	r6,r27
	shlq	#1,r4
	shlq	#1,r5
	shlq	#1,r6
.moved:
	store	r24,(r20)			;save the new coordinates
	store	r27,(r21)
	subq	#4,r20
	subq	#4,r21
	store	r23,(r20)
	store	r26,(r21)
  	subq	#4,r20
  	subq	#4,r21
  	store	r22,(r20)
  	store	r25,(r21)

  	jump	t,(r31)				;return from subroutine
  	nop

  	.68000

endobjcoll:

******************************** Data Section ********************************

  	data

********************************* Variables **********************************

  	even

gpuobjcoll:
	dc.l	objcoll,gpuprog,endobjcoll-objcoll
