******************************** GPU Routines ********************************

* Notes:

* ALL GPU programs should be placed on a phrase boundary
* because the blitter copies the code to GPU RAM in phrases

* All variables for GPU programs should be defined in 'equates.s'
* This is to prevent clashes when changing programs

******************************** Code Section ********************************

		text
		.68000

**************************** GPU Handling Routines ***************************

rungpu:		bsr.s	copygpu			;run a GPU program
		bsr.s	startgpu
		rts

copygpu:	moveq	#8,d0
		move.l	d0,G_CTRL
		cmp.l	gpu_routine_addr,a6
		bne.s	.must_copy
		rts
.must_copy:
		move.l	a6,gpu_routine_addr

		bsr.s	waitgpu
		bsr.s	waitblitter

		movem.l	(a6),d0/d1/d2
		move.l	d0,A2_BASE		;copy source into
		move.l	d1,A1_BASE		;GPU internal RAM
		
		lsr.w	#1,d2			;size in bytes
		addq.l	#1,d2
		or.l	#$10000,d2
		move.l	d2,B_COUNT

		moveq.l	#0,d0
		move.l	d0,A1_PIXEL
		move.l	d0,A2_PIXEL

		move.l	#PIXEL16|PITCH1|WID2048|XADDPHR,d0
		move.l	d0,A1_FLAGS
		move.l	d0,A2_FLAGS

		move.l	#LFU_REPLACE|SRCEN,B_CMD
.waitb2:
		rts

startgpu:	moveq	#1,d0
		move.l	d0,semaphore
		move.l	4(a6),G_PC		;run program that is
		move.l	d0,G_CTRL		;already in GPU memory
		rts

waitgpu:	move.l	G_CTRL,d0		;wait for GPU
		lsr.w	d0			;to stop
		bcc.s	.exit
		move.l	semaphore,d0
		bne.s	waitgpu
.exit:		rts

stop_gpu:	move.l	#8,G_CTRL
		rts

********************************** BSS Section *******************************

		bss

*********************************** Variables ********************************

		even

gpu_routine_addr:
		ds.l	1
